/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.configuration;

import org.apache.commons.configuration.XMLConfiguration;
import org.springframework.beans.factory.config.AbstractFactoryBean;
import org.springframework.core.io.Resource;

/**
 * Provides a <a href="http://jakarta.apache.org/commons/configuration/">Commons
 * Configuration</a> bean that enables the retrieval of application data.
 * <p/>
 * <p>The configuration data is drawn from the location specified at
 * <code>configLocation</code> which can be from the classpath, Web, etc.
 * however it is recommended to specify a file system location so that the data
 * can be dynamically reloaded via the {@link ConfigurationController}.</p>
 *
 * @author Juan Hall
 */
public class CommonsConfigurationFactoryBean extends AbstractFactoryBean {

  public void afterPropertiesSet() throws Exception {
    if (this.configLocation == null || !this.configLocation.exists()) {
      throw new IllegalArgumentException(
          "The given configuration location: '" + configLocation +
              "' is invalid");
    }

    super.afterPropertiesSet();
  }

  protected Object createInstance() throws Exception {
    return new XMLConfiguration(configLocation.getFile().getAbsolutePath());
  }

  public Class getObjectType() {
    return XMLConfiguration.class;
  }

  public boolean isSingleton() {
    return true;
  }

  private Resource configLocation;

  /**
   * Sets the location of the configuration file. A typical location is in
   * <code>WEB-INF/configuration.xml.${ENV}</code>
   *
   * @param configLocation location of the configuration file
   */
  public void setConfigLocation(Resource configLocation) {
    this.configLocation = configLocation;
  }
}
