/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.configuration;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import midstream.framework.logging.EventLogger;
import org.apache.commons.configuration.XMLConfiguration;
import org.springframework.beans.factory.BeanNameAware;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.multiaction.MultiActionController;

/**
 * Provides an HTTP endpoint for managing configuration resources. The primary
 * functionality is to expose a URL for reloading the application's
 * configuration data.
 *
 * @author Juan Hall
 */
public class ConfigurationController extends MultiActionController
    implements BeanNameAware {

  private String beanName;

  public void setBeanName(String beanName) {
    this.beanName = beanName;
  }

  private XMLConfiguration configuration;

  public void setConfiguration(XMLConfiguration configuration) {
    this.configuration = configuration;
  }

  /**
   * Reloads the configuration bean.
   *
   * @param request  the <code>HttpServletRequest</code>
   * @param response the <code>HttpServletResponse</code>
   * @return null
   */
  public ModelAndView reload(HttpServletRequest request,
                             HttpServletResponse response) {
    EventLogger log = new EventLogger(this.beanName + ".reload");
    String result = "The configuration has been reloaded successfully.";

    try {
      log.logBegin();
      this.configuration.clear();
      this.configuration.load(this.configuration.getFileName());
    } catch (Throwable th) {
      log.logError(th);

      StringWriter sr = new StringWriter();
      th.printStackTrace(new PrintWriter(sr));

      result = String.format("Unable to reload configuration due to: %s",
          sr.getBuffer().toString());
    } finally {
      try {
        response.getWriter().println(result);
      } catch (IOException e) {
        // Ignore this
      }

      log.logEnd();
    }

    return null;
  }
}
