/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.monitoring;

import com.jamonapi.MonitorFactory;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.AbstractController;

/**
 * Provides a Web endpoint for viewing the application's performance metrics.
 * The metrics are gathered by the <a href="http://www.javaperformancetuning.com/tools/jamon/index.shtml">JAMon</a>
 * API utilized by the {@link midstream.framework.logging.EventLogger}.
 * <p/>
 * <p>A <code>MonitorSummary</code> URL is automatically exposed for the
 * application at <code>http://&lt;host&gt;:&lt;port&gt;/&lt;deployment
 * prefix&gt;-&lt;deployment version&gt;/remote/MonitorSummary</code></p>
 *
 * @author Juan Hall
 */
public class MonitoringController extends AbstractController {
  protected ModelAndView handleRequestInternal(
      HttpServletRequest httpServletRequest,
      HttpServletResponse httpServletResponse) throws Exception {
    httpServletResponse.getWriter().write(MonitorFactory.getReport());
    return null;
  }
}
