/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.remoting.client;

import java.lang.reflect.Method;
import midstream.framework.logging.EventLogger;
import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.apache.commons.httpclient.HttpClient;
import org.springframework.aop.framework.ProxyFactoryBean;
import org.springframework.aop.support.StaticMethodMatcherPointcutAdvisor;
import org.springframework.beans.factory.BeanNameAware;
import org.springframework.beans.factory.InitializingBean;

/**
 * @author Juan Hall
 */
public class HttpClientProxyFactoryBean extends ProxyFactoryBean implements
    InitializingBean, BeanNameAware {

  public void afterPropertiesSet() throws Exception {
    MethodInterceptor interceptor = new MethodInterceptor() {
      public Object invoke(MethodInvocation methodInvocation) throws Throwable {
        EventLogger log = new EventLogger(
            beanName + "." + methodInvocation.getMethod().getName());

        try {
          log.logBegin();
          return methodInvocation.proceed();
        } catch (Throwable th) {
          log.logError(th);
          throw th;
        } finally {
          log.logEnd();
        }
      }
    };

    this.addAdvisor(new StaticMethodMatcherPointcutAdvisor(interceptor) {
      private static final long serialVersionUID = 1L;

      public boolean matches(Method method, Class clazz) {
        return "executeMethod".equals(method.getName());
      }
    });

    final HttpClient httpClient = ((HttpClient) this.getTargetSource()
        .getTarget());
    httpClient.getHttpConnectionManager().getParams().setConnectionTimeout(
        this.connectionTimeout * 1000);
    httpClient.getHttpConnectionManager().getParams().setSoTimeout(
        this.readTimeout * 1000);
  }

  private String beanName;

  public void setBeanName(String name) {
    this.beanName = name;
  }

  private int connectionTimeout = 10;

  public void setConnectionTimeout(int connectionTimeout) {
    this.connectionTimeout = connectionTimeout;
  }

  private int readTimeout = 10;

  public void setReadTimeout(int readTimeout) {
    this.readTimeout = readTimeout;
  }
}