/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.remoting.client;

import java.io.IOException;

/**
 * Specifies the API for sending requests to a service via a simple socket.
 *
 * @author Juan Hall
 */
public interface SocketClient {

  /**
   * Utilizes a <code>SocketClientDelegate</code> callback to for handling
   * communication with the underlying socket.
   *
   * @param delegate the callback to use for reading/writing to the socket
   * @throws IOException if an error occurs
   */
  void execute(SocketClientDelegate delegate) throws IOException;

  /**
   * Writes the given bytes to the socket and returns the response as an array
   * of bytes.
   *
   * @param bytes the request represented as an array of bytes
   * @return the response formatted as an array of bytes
   * @throws IOException if an error occurs
   */
  byte[] execute(final byte[] bytes) throws IOException;
}