/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.remoting.client;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * Specifies the API for outputting the request to a socket and reading the
 * resulting response.
 *
 * @author Juan Hall
 */
public interface SocketClientDelegate {

  /**
   * Writes the callback implementation's request to the given socket's output
   * stream.
   *
   * @param os the socket's output stream
   * @throws IOException if an error occurs
   */
  void writeRequest(OutputStream os) throws IOException;

  /**
   * Reads the service's response from the socket's input stream.
   *
   * @param is the socket's input stream to read the response from
   * @throws IOException if an error occurs
   */
  void readResponse(InputStream is) throws IOException;
}
