/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.remoting.httpinvoker;

import java.io.IOException;
import java.net.HttpURLConnection;
import midstream.framework.logging.EventLogger;
import org.springframework.remoting.httpinvoker.SimpleHttpInvokerRequestExecutor;
import org.springframework.remoting.support.RemoteInvocation;
import org.springframework.remoting.support.RemoteInvocationResult;

/**
 * @author Juan Hall
 */
public class HttpInvokerProxyFactoryBean extends
    org.springframework.remoting.httpinvoker.HttpInvokerProxyFactoryBean {

  private int connectionTimeout = 10000;
  private int readTimeout = 10000;

  public void afterPropertiesSet() {
    setHttpInvokerRequestExecutor(new SimpleHttpInvokerRequestExecutor() {
      protected void prepareConnection(HttpURLConnection httpURLConnection,
                                       int i)
          throws IOException {
        httpURLConnection.setConnectTimeout(connectionTimeout);
        httpURLConnection.setReadTimeout(readTimeout);

        super.prepareConnection(httpURLConnection, i);
      }
    });

    super.afterPropertiesSet();
  }

  @Override
  protected RemoteInvocationResult executeRequest(
      RemoteInvocation remoteInvocation) throws Exception {
    EventLogger log = new EventLogger(this.getServiceInterface()
        .getSimpleName() + "." + remoteInvocation.getMethodName());

    try {
      log.logBegin();
      return super.executeRequest(remoteInvocation);
    } catch (Exception e) {
      log.logError(e);
      throw e;
    } finally {
      log.logEnd();
    }
  }

  public void setConnectionTimeout(int connectionTimeout) {
    this.connectionTimeout = connectionTimeout;
  }

  public void setReadTimeout(int readTimeout) {
    this.readTimeout = readTimeout;
  }
}
