/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.remoting.xmlinvoker;

import java.lang.reflect.Method;
import midstream.framework.logging.EventLogger;
import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.codehaus.xfire.spring.remoting.XFireExporter;
import org.springframework.aop.framework.ProxyFactory;
import org.springframework.aop.support.StaticMethodMatcherPointcutAdvisor;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * @author Juan Hall
 */
public class SoapInvokerServiceExporter extends XFireExporter {

  @Override
  public void setApplicationContext(ApplicationContext context)
      throws BeansException {
    ClassPathXmlApplicationContext newContext =
        new ClassPathXmlApplicationContext(
            new String[]{"classpath:org/codehaus/xfire/spring/xfire.xml"},
            context);

    super.setApplicationContext(newContext);
  }

  @Override
  protected Object getProxyForService() {
    MethodInterceptor interceptor = new MethodInterceptor() {
      public Object invoke(MethodInvocation invocation) throws Throwable {
        EventLogger log = new EventLogger(getServiceClass()
            .getSimpleName() + "." + invocation.getMethod().getName());

        try {
          log.logBegin();
          return invocation.proceed();
        } catch (Throwable th) {
          log.logError(th);
          throw th;
        } finally {
          log.logEnd();
        }
      }
    };

    StaticMethodMatcherPointcutAdvisor advisor =
        new StaticMethodMatcherPointcutAdvisor(
            interceptor) {
          private static final long serialVersionUID = 1L;

          public boolean matches(Method method, Class arg1) {
            return true;
          }
        };

    ProxyFactory proxyFactory = new ProxyFactory(getServiceBean());
    proxyFactory.addInterface(getXFireService().getServiceInfo()
        .getServiceClass());
    proxyFactory.addAdvisor(advisor);

    return proxyFactory.getProxy();
  }

  public void setService(Object bean) {
    setServiceBean(bean);
  }
}