/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.remoting.xmlinvoker;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import midstream.framework.logging.EventLogger;
import org.apache.xmlrpc.XmlRpcException;
import org.apache.xmlrpc.XmlRpcRequest;
import org.apache.xmlrpc.server.PropertyHandlerMapping;
import org.apache.xmlrpc.server.RequestProcessorFactoryFactory;
import org.apache.xmlrpc.webserver.XmlRpcServletServer;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.remoting.support.RemoteExporter;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.Controller;

/**
 * @author Juan Hall
 */
public class XmlRpcInvokerServiceExporter extends RemoteExporter implements
    Controller, InitializingBean {

  XmlRpcServletServer rpcServer = new XmlRpcServletServer() {
    @Override
    public Object execute(XmlRpcRequest request) throws XmlRpcException {
      String serviceId = request.getMethodName();
      EventLogger log = new EventLogger(serviceId);

      try {
        log.logBegin();
        return super.execute(request);
      } catch (Throwable th) {
        log.logError(th);
        XmlRpcException e = new XmlRpcException(-32500,
            "Error invoking XML-RPC exporter: " + serviceId);
        e.initCause(th);
        throw e;
      } finally {
        log.logEnd();
      }
    }
  };

  public void afterPropertiesSet() throws Exception {
    PropertyHandlerMapping mapping = new PropertyHandlerMapping();
    mapping
        .setRequestProcessorFactoryFactory(
            new RequestProcessorFactoryFactory.RequestSpecificProcessorFactoryFactory() {
              @Override
              protected Object getRequestProcessor(Class arg0,
                                                   XmlRpcRequest arg1) {
                return getProxyForService();
              }
            });
    mapping.addHandler(getServiceInterface().getSimpleName(),
        getServiceInterface());
    rpcServer.setHandlerMapping(mapping);
  }

  public ModelAndView handleRequest(HttpServletRequest request,
                                    HttpServletResponse response)
      throws Exception {
    rpcServer.execute(request, response);
    return null;
  }
}