/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.scheduling.timer;

import java.lang.reflect.Method;
import midstream.framework.logging.EventLogger;
import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.springframework.aop.framework.ProxyFactory;
import org.springframework.aop.support.StaticMethodMatcherPointcutAdvisor;
import org.springframework.beans.factory.BeanNameAware;

/**
 * @author Juan Hall
 */
public class MethodInvokingTimerTaskFactoryBean extends
    org.springframework.scheduling.timer.MethodInvokingTimerTaskFactoryBean
    implements BeanNameAware {

  private String beanName;

  public void setBeanName(String name) {
    this.beanName = name;
  }

  @Override
  public void afterPropertiesSet()
      throws ClassNotFoundException, NoSuchMethodException {
    if (this.getTargetObject() == null) {
      throw new IllegalArgumentException("A target object is required");
    }

    MethodInterceptor interceptor = new MethodInterceptor() {
      public Object invoke(MethodInvocation methodInvocation) throws Throwable {
        EventLogger log = new EventLogger(
            beanName + "." + methodInvocation.getMethod().getName());

        try {
          log.logBegin();
          return methodInvocation.proceed();
        } catch (Throwable th) {
          log.logError(th);
          throw th;
        } finally {
          log.logEnd();
        }
      }
    };

    ProxyFactory factory = new ProxyFactory(this.getTargetObject());
    factory.addAdvisor(new StaticMethodMatcherPointcutAdvisor(interceptor) {
      public boolean matches(Method method, Class aClass) {
        return method.getName().equals(getTargetMethod());
      }
    });

    this.setTargetObject(factory.getProxy());

    super.afterPropertiesSet();
  }
}
