/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package midstream.framework;

import java.io.PrintWriter;
import java.io.StringWriter;
import junit.framework.TestCase;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.WriterAppender;
import org.junit.AfterClass;
import org.junit.BeforeClass;

/**
 * Base class for test classes invoking components that perform event logging.
 *
 * @author Juan Hall
 */
public abstract class LoggingTestCase extends TestCase {

  protected StringWriter logBuffer;

  @BeforeClass
  @Override
  protected void setUp() throws Exception {
    super.setUp();

    logBuffer = new StringWriter();

    Logger.getLogger("event-logger").setLevel(Level.INFO);
    Logger.getLogger("event-logger").addAppender(new WriterAppender(
        new PatternLayout("%m%n"), new PrintWriter(logBuffer)));
  }

  @AfterClass
  @Override
  protected void tearDown() throws Exception {
    Logger.getLogger("event-logger").removeAllAppenders();
    logBuffer = null;

    super.tearDown();
  }
}
