/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.configuration;

import junit.framework.TestCase;
import org.apache.commons.configuration.XMLConfiguration;
import org.junit.Test;
import org.springframework.core.io.FileSystemResourceLoader;

/**
 * @author Juan Hall
 */
public class CommonsConfigurationFactoryBeanTest extends TestCase {

  @Test
  public void testAfterPropertiesSetWithoutConfigLocation() {
    try {
      new CommonsConfigurationFactoryBean().afterPropertiesSet();
      fail("An IllegalStateException should have been thrown!");
    } catch (Exception e) {
      assertTrue(e instanceof IllegalArgumentException);
      assertTrue(e.getMessage().matches(
          ".* configuration location: '.*' is invalid"));
    }
  }

  @Test
  public void testCreateInstance() throws Exception {
    CommonsConfigurationFactoryBean factoryBean =
        new CommonsConfigurationFactoryBean();
    factoryBean.setConfigLocation(
        new FileSystemResourceLoader().getResource("test/src/" +
            this.getClass().getPackage().getName().replace('.', '/') +
            "/test-config.xml"));
    XMLConfiguration config = (XMLConfiguration) factoryBean.createInstance();
    assertEquals("value", config.getString("key"));
  }

  @Test
  public void testGetObjectType() {
    assertTrue(XMLConfiguration.class.isAssignableFrom(
        new CommonsConfigurationFactoryBean().getObjectType()));
  }
}
