/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.configuration;

import com.mockobjects.dynamic.Mock;
import java.io.PrintWriter;
import java.io.StringWriter;
import javax.servlet.http.HttpServletResponse;
import junit.framework.TestCase;
import org.apache.commons.configuration.XMLConfiguration;
import org.junit.Test;

/**
 * @author Juan Hall
 */
public class ConfigurationControllerTest extends TestCase {

  @Test
  public void testReload() throws Exception {
    ConfigurationController ctrlr = new ConfigurationController();
    XMLConfiguration config = new XMLConfiguration("file:test/src/" +
        this.getClass().getPackage().getName().replace('.', '/') +
        "/test-config.xml");
    ctrlr.setConfiguration(config);

    config.setProperty("key", "test");
    assertEquals("test", config.getString("key"));

    StringWriter sr = new StringWriter();
    Mock responseMock = new Mock(HttpServletResponse.class);
    responseMock.expectAndReturn("getWriter", new PrintWriter(sr));

    ctrlr.reload(null, (HttpServletResponse) responseMock.proxy());

    assertEquals("value", config.getString("key"));
    assertEquals("The configuration has been reloaded successfully." +
        System.getProperty("line.separator"), sr.getBuffer().toString());

    responseMock.verify();
  }
}
