/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package midstream.framework.jdbc;

import midstream.framework.LoggingTestCase;
import org.junit.Test;

/**
 * @author Juan Hall
 */
public class JdbcDaoSupportProxyFactoryBeanTest extends LoggingTestCase {

  @Test
  public void testAfterPropertiesSetWithoutTargetInterface() {
    try {
      new JdbcDaoSupportProxyFactoryBean().afterPropertiesSet();
      fail("An IllegalStateException should have been thrown!");
    } catch (Exception e) {
      assertTrue(e instanceof IllegalArgumentException);
      assertTrue(e.getMessage().matches(
          ".* target interface must be specified"));
    }
  }

  @Test
  public void testAfterPropertiesSet() throws Exception {
    TestDao dao = new TestDao() {
      public String execute(String request) {
        return request;
      }
    };
    JdbcDaoSupportProxyFactoryBean factoryBean =
        new JdbcDaoSupportProxyFactoryBean();
    factoryBean.setTargetInterface(TestDao.class);
    factoryBean.setTarget(dao);
    factoryBean.afterPropertiesSet();

    assertEquals("test", ((TestDao) factoryBean.getObject()).execute("test"));

    assertTrue(logBuffer.getBuffer().indexOf("BEG|") != -1);
    assertTrue(logBuffer.getBuffer().indexOf("ERR|") == -1);
    assertTrue(logBuffer.getBuffer().indexOf("END|") != -1);
  }
}

interface TestDao {
  String execute(String request);
}