/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package midstream.framework.jms.listener;

import com.mockobjects.dynamic.Mock;
import javax.jms.ConnectionFactory;
import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import midstream.framework.LoggingTestCase;
import org.junit.Test;

/**
 * @author Juan Hall
 */
public class DefaultMessageListenerContainerTest extends LoggingTestCase {

  @Test
  public void testAfterPropertiesSet() {
    TestReceiver receiver = new TestReceiver();
    Mock connectionFactoryMock = new Mock(ConnectionFactory.class);
    connectionFactoryMock.matchAndReturn("createConnection", null);

    final Object[] listeners = new Object[2];
    DefaultMessageListenerContainer container =
        new DefaultMessageListenerContainer() {
          @Override
          public void setMessageListener(Object object) {
            super.setMessageListener(object);
            listeners[0] = object;
          }

          @Override
          public void setExceptionListener(
              ExceptionListener exceptionListener) {
            super.setExceptionListener(exceptionListener);
            listeners[1] = exceptionListener;
          }
        };
    container.setMessageListener(receiver);
    container.setExceptionListener(receiver);
    container.setBeanName("test");
    container.setConnectionFactory(
        (ConnectionFactory) connectionFactoryMock.proxy());
    container.setDestinationName("test");
    container.afterPropertiesSet();

    ((MessageListener) listeners[0]).onMessage(null);
    ((ExceptionListener) listeners[1]).onException(null);

    assertTrue(logBuffer.getBuffer().indexOf("|test.onMessage") != -1);
    assertTrue(logBuffer.getBuffer().indexOf("|test.onException") != -1);
    assertTrue(logBuffer.getBuffer().indexOf("ERR|") == -1);

    connectionFactoryMock.verify();
  }
}

class TestReceiver implements MessageListener, ExceptionListener {
  public void onMessage(Message message) {
  }

  public void onException(JMSException jmsException) {
  }
}