/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package midstream.framework.logging;

import com.jamonapi.MonitorFactory;
import midstream.framework.LoggingTestCase;
import org.junit.Test;

/**
 * @author Juan Hall
 */
public class EventLoggerTest extends LoggingTestCase {

  @Test
  public void testLogBegin() {
    new EventLogger("test") {
      @Override
      long getStartTime() {
        return 1;
      }

      @Override
      long getThreadId() {
        return 1;
      }
    }.logBegin();

    assertEquals("BEG|1-1|test" + System.getProperty("line.separator"),
        logBuffer.getBuffer().toString());
  }

  @Test
  public void testLogError() {
    EventLogger log = new EventLogger("test") {
      @Override
      long getThreadId() {
        return 1;
      }
    };
    log.startTime = 1;
    log.monitor = MonitorFactory.start();
    log.logError(new Exception("message"));

    assertTrue(logBuffer.getBuffer().toString().startsWith(
        "ERR|1-1|test|message" + System.getProperty("line.separator")));
  }

  @Test
  public void testLogEnd() {
    EventLogger log = new EventLogger("test") {
      @Override
      long getThreadId() {
        return 1;
      }
    };
    log.startTime = 1;
    log.monitor = MonitorFactory.start();

    assertEquals(0.0, log.monitor.getHits());

    log.logEnd();

    assertEquals("END|1-1|test" + System.getProperty("line.separator"),
        logBuffer.getBuffer().toString());
    assertEquals(1.0, log.monitor.getHits());
  }
}
