/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.remoting.client;

import midstream.framework.LoggingTestCase;
import org.apache.axis.client.Stub;
import org.junit.Test;

/**
 * @author Juan Hall
 */
public class AxisClientProxyFactoryBeanTest extends LoggingTestCase {

  @Test
  public void testAfterPropertiesSet() throws Exception {
    AxisClientProxyFactoryBean factoryBean = new AxisClientProxyFactoryBean();
    factoryBean.setBeanName("testBean");
    factoryBean.setTarget(new TestServiceLocator());
    factoryBean.setTimeout(10);
    factoryBean.afterPropertiesSet();

    TestServiceLocator testServiceLocator =
        ((TestServiceLocator) factoryBean.getObject());
    assertEquals(10000, testServiceLocator.getTestPort().getTimeout());
    testServiceLocator.getTestPort().execute();

    assertTrue(logBuffer.getBuffer().indexOf("BEG|") != -1);
    assertTrue(logBuffer.getBuffer().indexOf("ERR|") == -1);
    assertTrue(logBuffer.getBuffer().indexOf("END|") != -1);
  }
}

class TestPort extends Stub {
  public int timeout = -1;

  @Override
  public int getTimeout() {
    return timeout;
  }

  @Override
  public void setTimeout(int i) {
    this.timeout = i;
  }

  public void execute() {
    System.out.println("hello");
  }
}

class TestServiceLocator {
  private TestPort port = new TestPort();

  public TestPort getTestPort() {
    return port;
  }
}
