/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package midstream.framework.remoting.client;

import java.io.IOException;
import midstream.framework.LoggingTestCase;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpMethod;
import org.junit.Test;

/**
 * @author Juan Hall
 */
public class HttpClientProxyFactoryBeanTest extends LoggingTestCase {

  @Test
  public void testAfterPropertiesSet() throws Exception {
    HttpClientProxyFactoryBean factoryBean = new HttpClientProxyFactoryBean();
    factoryBean.setTarget(new TestHttpClient());
    factoryBean.setConnectionTimeout(10);
    factoryBean.setReadTimeout(8);
    factoryBean.afterPropertiesSet();

    HttpClient httpClient = ((HttpClient) factoryBean.getObject());

    assertEquals(10000, httpClient.getHttpConnectionManager()
        .getParams().getConnectionTimeout());
    assertEquals(8000,
        httpClient.getHttpConnectionManager().getParams().getSoTimeout());

    httpClient.executeMethod(null);

    assertTrue(logBuffer.getBuffer().indexOf("BEG|") != -1);
    assertTrue(logBuffer.getBuffer().indexOf("ERR|") == -1);
    assertTrue(logBuffer.getBuffer().indexOf("END|") != -1);
  }
}

class TestHttpClient extends HttpClient {
  @Override
  public int executeMethod(HttpMethod httpMethod)
      throws IOException, HttpException {
    return -1;
  }
}