/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.framework.remoting.client;

import junit.framework.TestCase;
import org.junit.Test;
import org.springframework.context.support.AbstractXmlApplicationContext;
import org.springframework.context.support.FileSystemXmlApplicationContext;

/**
 * @author Juan Hall
 */
public class ServiceLocatorTest extends TestCase {

  @Test
  public void testGetService() throws Exception {
    ServiceLocator.context = new FileSystemXmlApplicationContext(
        "test/src/" +
            this.getClass().getPackage().getName().replace('.', '/') +
            "/testContext.xml");

    assertTrue(ServiceLocator.getService("testBean") instanceof TestBean);
  }

  @Test
  public void testGetServiceWithInvalidBeanName() {
    ServiceLocator.context = new FileSystemXmlApplicationContext(
        "test/src/" +
            this.getClass().getPackage().getName().replace('.', '/') +
            "/testContext.xml");

    try {
      ServiceLocator.getService("invalidBean");
      fail("A ServiceException should have been thrown");
    } catch (ServiceLocator.ServiceException e) {
      assertEquals("Unable to retrieve service: invalidBean", e.getMessage());
    }
  }

  @Test
  public void testGetServiceUrl() throws Exception {
    ServiceLocator.context = new FileSystemXmlApplicationContext(
        "test/src/" +
            this.getClass().getPackage().getName().replace('.', '/') +
            "/testContext.xml");

    assertEquals("http://localhost:9999/path",
        ServiceLocator.getServiceUrl("testBean"));
  }

  @Test
  public void testSetServiceUrl() throws Exception {
    AbstractXmlApplicationContext oldCtx = ServiceLocator.context;
    try {
      ServiceLocator.context = new FileSystemXmlApplicationContext(
          "test/src/" +
              this.getClass().getPackage().getName().replace('.', '/') +
              "/testContext.xml");

      assertEquals("http://localhost:9999/path", ((TestBean) ServiceLocator
          .getService("testBean")).getServiceUrl());

      ServiceLocator.setServiceUrl("testBean", "http://anotherhost:1234/page");

      assertEquals("http://anotherhost:1234/page", ((TestBean) ServiceLocator
          .getService("testBean")).getServiceUrl());
    } finally {
      ServiceLocator.context = oldCtx;
    }
  }
}
