/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package midstream.framework.remoting.client;

import midstream.framework.LoggingTestCase;
import org.apache.xmlrpc.XmlRpcException;
import org.apache.xmlrpc.XmlRpcRequest;
import org.apache.xmlrpc.client.XmlRpcClient;
import org.apache.xmlrpc.client.XmlRpcClientConfigImpl;
import org.junit.Test;

/**
 * @author Juan Hall
 */
public class XmlRpcClientProxyFactoryBeanTest extends LoggingTestCase {

  @Test
  public void testAfterPropertiesSet() throws Exception {
    XmlRpcClientProxyFactoryBean factoryBean =
        new XmlRpcClientProxyFactoryBean();
    factoryBean.setTarget(new TestXmlRpcClient());
    factoryBean.setUrl("http://localhost");
    factoryBean.setConnectionTimeout(10);
    factoryBean.setReadTimeout(8);
    factoryBean.afterPropertiesSet();

    XmlRpcClient xmlRpcClient = ((XmlRpcClient) factoryBean.getObject());

    assertEquals(10000, ((XmlRpcClientConfigImpl) xmlRpcClient
        .getClientConfig()).getConnectionTimeout());
    assertEquals(8000, ((XmlRpcClientConfigImpl) xmlRpcClient
        .getClientConfig()).getReplyTimeout());

    xmlRpcClient.execute(null);

    assertTrue(logBuffer.getBuffer().indexOf("BEG|") != -1);
    assertTrue(logBuffer.getBuffer().indexOf("ERR|") == -1);
    assertTrue(logBuffer.getBuffer().indexOf("END|") != -1);

  }
}

class TestXmlRpcClient extends XmlRpcClient {

  @Override
  public Object execute(XmlRpcRequest xmlRpcRequest) throws XmlRpcException {
    return null;
  }
}