/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package midstream.framework.remoting.httpinvoker;

import java.io.ByteArrayOutputStream;
import java.io.Serializable;
import java.lang.reflect.Method;
import java.net.HttpURLConnection;
import java.net.URL;
import midstream.framework.LoggingTestCase;
import org.junit.Test;
import org.springframework.remoting.httpinvoker.HttpInvokerClientConfiguration;
import org.springframework.remoting.httpinvoker.SimpleHttpInvokerRequestExecutor;
import org.springframework.remoting.support.RemoteInvocation;
import org.springframework.remoting.support.RemoteInvocationResult;

/**
 * @author Juan Hall
 */
public class HttpInvokerProxyFactoryBeanTest extends LoggingTestCase {

  @Test
  public void testAfterPropertiesSet() throws Exception {
    HttpInvokerProxyFactoryBean factoryBean = new HttpInvokerProxyFactoryBean();
    factoryBean.setConnectionTimeout(5000);
    factoryBean.setReadTimeout(6000);
    factoryBean.setServiceUrl("http://localhost");
    factoryBean.setServiceInterface(Serializable.class);
    factoryBean.afterPropertiesSet();

    HttpURLConnection con = new HttpURLConnection(new URL("http://locahost")) {
      public void disconnect() {
      }

      public boolean usingProxy() {
        return false;
      }

      public void connect() {
      }
    };

    Method m = SimpleHttpInvokerRequestExecutor.class
        .getDeclaredMethod("prepareConnection", HttpURLConnection.class,
            int.class);
    m.setAccessible(true);
    m.invoke(factoryBean.getHttpInvokerRequestExecutor(), con, 0);

    assertEquals(5000, con.getConnectTimeout());
    assertEquals(6000, con.getReadTimeout());
  }

  @Test
  public void testExecuteRequest() throws Exception {
    HttpInvokerProxyFactoryBean factoryBean = new HttpInvokerProxyFactoryBean();
    factoryBean
        .setHttpInvokerRequestExecutor(new SimpleHttpInvokerRequestExecutor() {
          protected RemoteInvocationResult doExecuteRequest(
              HttpInvokerClientConfiguration httpInvokerClientConfiguration,
              ByteArrayOutputStream byteArrayOutputStream) {
            return null;
          }
        });
    factoryBean.setServiceInterface(Serializable.class);
    RemoteInvocation invocation = new RemoteInvocation("toString", null, null);

    factoryBean.executeRequest(invocation);

    assertTrue(logBuffer.getBuffer().indexOf("BEG|") != -1);
    assertTrue(logBuffer.getBuffer().indexOf("ERR|") == -1);
    assertTrue(logBuffer.getBuffer().indexOf("END|") != -1);
  }
}
