/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package midstream.framework.remoting.xmlinvoker;

import com.mockobjects.dynamic.C;
import com.mockobjects.dynamic.Mock;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import javax.servlet.ServletInputStream;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import midstream.framework.LoggingTestCase;
import org.junit.Test;

/**
 * @author Juan Hall
 */
public class XmlRpcInvokerServiceExporterTest extends LoggingTestCase {

  public interface ITestService {
    String execute(String request);
  }

  public class TestServiceImpl implements ITestService {
    public String execute(String request) {
      return "response";
    }
  }

  @Test
  public void testAfterPropertiesSet() throws Exception {
    XmlRpcInvokerServiceExporter exporter = new XmlRpcInvokerServiceExporter();
    exporter.setServiceInterface(ITestService.class);
    exporter.afterPropertiesSet();

    assertNotNull(exporter.rpcServer.getHandlerMapping().getHandler(
        ITestService.class.getSimpleName() + ".execute"));
  }

  @Test
  public void testHandleRequest() throws Exception {
    XmlRpcInvokerServiceExporter exporter = new XmlRpcInvokerServiceExporter();
    exporter.setServiceInterface(ITestService.class);
    exporter.setService(new TestServiceImpl());
    exporter.afterPropertiesSet();

    final StringReader reader =
        new StringReader("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
            "<methodCall><methodName>" + ITestService.class.getSimpleName() +
            ".execute</methodName><params><param>\n" +
            "<value><string>test</string></value>\n" +
            "</param></params></methodCall>");
    ServletInputStream inputStream = new ServletInputStream() {
      @Override
      public int read() throws IOException {
        return reader.read();
      }
    };

    Mock requestMock = new Mock(HttpServletRequest.class);
    requestMock.expectAndReturn("getHeader", "Content-Encoding", null);
    requestMock.expectAndReturn("getHeader", "Authorization", null);
    requestMock.expectAndReturn("getCharacterEncoding", null);
    requestMock.expectAndReturn("getInputStream", inputStream);
    requestMock.expectAndReturn("getHeaders", "Accept-Encoding", null);

    final StringWriter sr = new StringWriter();
    ServletOutputStream outputStream = new ServletOutputStream() {
      @Override
      public void write(int arg0) {
        sr.write(arg0);
      }
    };

    Mock responseMock = new Mock(HttpServletResponse.class);
    responseMock.expect("setContentLength", C.ANY_ARGS);
    responseMock.expect("setContentType", C.ANY_ARGS);
    responseMock.matchAndReturn("getOutputStream", outputStream);

    exporter.handleRequest((HttpServletRequest) requestMock.proxy(),
        (HttpServletResponse) responseMock.proxy());

    assertEquals(
        "<?xml version=\"1.0\" encoding=\"UTF-8\"?><methodResponse>" +
            "<params><param><value>response</value></param></params></methodResponse>",
        sr.getBuffer().toString());
    requestMock.verify();
    responseMock.verify();
  }
}