/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package midstream.framework.scheduling.timer;

import midstream.framework.LoggingTestCase;
import org.junit.Test;
import org.springframework.scheduling.timer.DelegatingTimerTask;

/**
 * @author Juan Hall
 */
public class MethodInvokingTimerTaskFactoryBeanTest extends LoggingTestCase {

  @Test
  public void testAfterPropertiesSetWithoutValidArguments() throws Exception {
    MethodInvokingTimerTaskFactoryBean factoryBean =
        new MethodInvokingTimerTaskFactoryBean();

    try {
      factoryBean.afterPropertiesSet();
      fail("An IllegalArgumentException should have been thrown");
    } catch (Exception e) {
      assertTrue(e instanceof IllegalArgumentException);
      assertEquals("A target object is required", e.getMessage());
    }
  }

  @Test
  public void testAfterPropertiesSetWithValidArguments() throws Exception {
    MethodInvokingTimerTaskFactoryBean factoryBean =
        new MethodInvokingTimerTaskFactoryBean();
    factoryBean.setTargetMethod("execute");
    factoryBean.setTargetObject(new TestService());
    factoryBean.afterPropertiesSet();

    ((DelegatingTimerTask) factoryBean.getObject()).run();

    assertTrue(logBuffer.getBuffer().indexOf("BEG|") != -1);
    assertTrue(logBuffer.getBuffer().indexOf("ERR|") == -1);
    assertTrue(logBuffer.getBuffer().indexOf("END|") != -1);
  }
}

class TestService {
  public void execute() {
  }
}