/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package midstream.core.handler;

import midstream.core.util.WizardUtils;
import org.dom4j.Document;
import org.dom4j.Namespace;

import java.io.File;
import java.io.IOException;
import java.io.FileInputStream;

/**
 * Base class for all plugin action handlers.
 *
 * @author Juan Hall
 */
public abstract class BaseHandler {

  protected String projectPath;

  public BaseHandler(String projectPath) {
    this.projectPath = projectPath;
  }

  protected Document readApplicationContext() throws IOException {
    final File contextFile = new File(projectPath + File.separatorChar + "conf"
        + File.separatorChar + "applicationContext.xml");

    Document doc = WizardUtils.readXmlDocument(contextFile, false);

    if (doc == null) {
      throw new IOException("Unable to read the application's context file.");
    }

    doc.getRootElement().add(
        new Namespace("ns", "http://www.springframework.org/schema/beans"));

    return doc;
  }

  protected void outputApplicationContext(Document doc) {
    final File contextFile = new File(projectPath + File.separatorChar + "conf"
        + File.separatorChar + "applicationContext.xml");

    WizardUtils.outputXmlDocument(contextFile, doc);
  }

  protected String readFile(final String fileName) throws IOException {
    return WizardUtils.readStream(new FileInputStream(fileName));
  }

  protected void writeFile(final String fileName, final String contents,
                           final boolean overwrite) throws IOException {
    WizardUtils.writeFile(fileName, contents, true);
  }
}
