/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core.handler;

import midstream.core.util.WizardUtils;
import org.dom4j.Document;
import org.dom4j.Element;
import org.springframework.util.FileCopyUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

/**
 * @author Juan Hall
 */
public class CacheManagerBeanHandler extends BaseHandler {

  public static final String[] LIBRARIES =
      {"net.sf.ehcache:ehcache", "commons-collections:commons-collections"};

  public CacheManagerBeanHandler(String projectPath) {
    super(projectPath);
  }

  public void createCacheConfigFiles()
      throws IOException {
    String path = projectPath + File.separatorChar + "conf";
    new File(path).mkdirs();

    FileCopyUtils.copy(CacheManagerBeanHandler.class
        .getClassLoader()
        .getResourceAsStream(
        "midstream/core/templates/components/ehcache.xml"),
        new FileOutputStream(path + File.separatorChar + "ehcache.xml"));
  }

  public void createCacheFactoryBeanClassTest()
      throws IOException {
    WizardUtils
        .copyFile(
            CacheManagerBeanHandler.class
                .getClassLoader()
                .getResourceAsStream(
                "midstream/core/templates/components/CacheManagerBeanClassTest.template"),
            projectPath + File.separatorChar + "test" + File.separatorChar
                + "src" + File.separatorChar
                + "CacheManagerBeanTest.java", false);
  }

  public void createServiceContextEntry()
      throws IOException {
    Document doc = readApplicationContext();

    if (doc == null) {
      throw new IOException("Unable to read the application's context file");
    }

    Element root = doc.getRootElement();

    if (doc
        .selectSingleNode(
            "//ns:bean[@id='cacheManager']") ==
        null) {
      root.addElement("bean").addAttribute("id",
          "cacheManager")
          .addAttribute("class",
              "org.springframework.cache.ehcache.EhCacheManagerFactoryBean")
          .addElement("property").addAttribute("name", "configLocation")
          .addAttribute(
              "value",
              "${CONF_DIR}/ehcache.xml");
    }

    outputApplicationContext(doc);
  }

  public void updateBuildFile()
      throws IOException {
    File buildFile = new File(projectPath + File.separatorChar + "build.xml");
    Document doc = WizardUtils.readXmlDocument(buildFile, false);

    if (doc == null) {
      throw new IOException("Unable to read the 'build.xml'");
    }

    Element node = (Element) doc.selectSingleNode("//webinf[@dir='conf']");

    if (node != null) {
      node.addAttribute("includes",
          node.attributeValue("includes") + ", ehcache.xml");
      WizardUtils.outputXmlDocument(buildFile, doc);
    }
  }
}
