/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core.handler;

import midstream.core.util.WizardUtils;
import org.dom4j.Document;
import org.dom4j.Element;

import java.io.File;
import java.io.IOException;

/**
 * @author Juan Hall
 */
public class ConfigurationBeanHandler extends BaseHandler {

  public static final String[] LIBRARIES =
      {"commons-configuration:commons-configuration",
          "commons-lang:commons-lang",
          "commons-collections:commons-collections"};

  public ConfigurationBeanHandler(String projectPath) {
    super(projectPath);
  }

  public void createConfigurationBeanClassTest()
      throws IOException {
    WizardUtils
        .copyFile(
            ConfigurationBeanHandler.class
                .getClassLoader()
                .getResourceAsStream(
                "midstream/core/templates/components/ConfigurationBeanClassTest.template"),
            projectPath + File.separatorChar + "test" + File.separatorChar
                + "src" + File.separatorChar
                + "ConfigurationBeanTest.java", false);
  }

  public void createConfigurationFiles()
      throws IOException {
    String path = projectPath + File.separatorChar + "conf";
    new File(path).mkdirs();

    WizardUtils
        .copyFile(
            ConfigurationBeanHandler.class
                .getClassLoader()
                .getResourceAsStream(
                "midstream/core/templates/components/configuration.xml"),
            path + File.separatorChar + "configuration.xml.Development", false);
    WizardUtils
        .copyFile(
            ConfigurationBeanHandler.class
                .getClassLoader()
                .getResourceAsStream(
                "midstream/core/templates/components/configuration.xml"),
            path + File.separatorChar + "configuration.xml.Test", false);
    WizardUtils
        .copyFile(
            ConfigurationBeanHandler.class
                .getClassLoader()
                .getResourceAsStream(
                "midstream/core/templates/components/configuration.xml"),
            path + File.separatorChar + "configuration.xml.Production", false);
  }

  public void createServiceContextEntry()
      throws IOException {
    Document doc = readApplicationContext();

    if (doc == null) {
      throw new IOException("Unable to read the application's context file");
    }

    Element root = doc.getRootElement();

    if (doc
        .selectSingleNode(
            "//ns:bean[@id='configuration']") !=
        null) {
      return;
    }

    WizardUtils.addPlaceholderConfig(doc, root);

    root.addElement("bean").addAttribute("id", "configuration")
        .addAttribute("class",
            "midstream.framework.configuration.CommonsConfigurationFactoryBean")
        .addElement("property").addAttribute("name", "configLocation")
        .addAttribute(
            "value",
            "${CONF_DIR}/configuration.xml.${MIDSTREAM_ENV}");

    addConfigurationControllerBean(root);

    outputApplicationContext(doc);
  }

  static void addConfigurationControllerBean(Element root) {
    if (root.selectSingleNode("//ns:bean[@id='paramResolver']") == null) {
      root.addElement("bean").addAttribute("id", "paramResolver")
          .addAttribute("class",
              "org.springframework.web.servlet.mvc.multiaction.ParameterMethodNameResolver")
          .addElement("property").addAttribute("name", "paramName")
          .addAttribute("value", "method");
    }

    if (root.selectSingleNode("//ns:bean[@name='/Configuration']") == null) {
      Element bean =
          root.addElement("bean").addAttribute("name", "/Configuration")
              .addAttribute("class",
                  "midstream.framework.configuration.ConfigurationController");
      bean
          .addElement("property").addAttribute("name", "configuration")
          .addAttribute("ref", "configuration");
      bean
          .addElement("property").addAttribute("name", "methodNameResolver")
          .addAttribute("ref", "paramResolver");
    }
  }

  public void updateBuildFile()
      throws IOException {
    File buildFile = new File(projectPath + File.separatorChar + "build.xml");
    Document doc = WizardUtils.readXmlDocument(buildFile, false);

    if (doc == null) {
      throw new IOException("Unable to read the 'build.xml'");
    }

    Element node = (Element) doc.selectSingleNode("//webinf[@dir='conf']");

    if (node != null) {
      node.addAttribute("includes",
          node.attributeValue("includes") + ", configuration.xml.*");
      WizardUtils.outputXmlDocument(buildFile, doc);
    }
  }
}
