/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core.handler;

import midstream.core.util.SourceUtils;
import midstream.core.util.WizardUtils;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.Namespace;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class DaoHandler extends BaseHandler {

  public static final String[] LIBRARIES = {"hsqldb:hsqldb"};

  public DaoHandler(String projectPath) {
    super(projectPath);
  }

  public void createServiceContextEntry(
      Map<String, String> pageValues)
      throws IOException {
    Document doc = readApplicationContext();

    if (doc == null) {
      throw new IOException("Unable to read the application's context file");
    }

    Element root = doc.getRootElement();

    if (doc.selectSingleNode("//ns:bean[@id='" + pageValues.get("dao.name")
        + "Target']") == null) {
      Element bean = root.addElement("bean");
      bean.addAttribute("id", pageValues.get("dao.name") + "Target")
          .addAttribute(
              "class",
              pageValues.get("java.package") + "." + pageValues.get("dao.name")
                  + "Impl");
      bean.addElement("property").addAttribute("name", "dataSource")
          .addAttribute("ref", pageValues.get("datasource"));
      bean.addElement("property").addAttribute("name", "sql").addElement(
          "value").addCDATA(pageValues.get("sql"));
    }

    if (doc
        .selectSingleNode(
            "//ns:bean[@id='" + pageValues.get("dao.name") + "']") ==
        null) {
      Element bean = root
          .addElement("bean")
          .addAttribute("id", pageValues.get("dao.name"))
          .addAttribute("class",
              "midstream.framework.jdbc.JdbcDaoSupportProxyFactoryBean");
      bean
          .addElement("property").addAttribute("name", "target")
          .addAttribute("ref", pageValues.get("dao.name") + "Target");
      bean
          .addElement("property").addAttribute("name", "targetInterface")
          .addAttribute("value", pageValues.get("java.package") + "." +
              pageValues.get("dao.name"));
    }

    if (doc.selectSingleNode("//ns:bean[@id='" + pageValues.get("datasource")
        + "']") == null) {
      root.addElement("bean").addAttribute("id", pageValues.get("datasource"))
          .addAttribute("class",
              "org.springframework.jndi.JndiObjectFactoryBean").addElement(
          "property").addAttribute("name", "jndiName").addAttribute(
          "value", "java:comp/env/" + pageValues.get("datasource"));
    }

    outputApplicationContext(doc);
  }

  public void createDaoClassTest(
      Map<String, String> pageValues)
      throws IOException {
    Map<String, String> stubs = new HashMap<String, String>(2);
    stubs.put("$package$", pageValues.get("java.package"));
    stubs.put("$name$", pageValues.get("dao.name"));

    SourceUtils.createSourceClassTest(projectPath, pageValues.get("dao.name")
        + "Impl", stubs, "DaoClass");
  }

  public String createDaoClass(
      Map<String, String> pageValues)
      throws IOException {
    Map<String, String> stubs = new HashMap<String, String>(2);
    stubs.put("$package$", pageValues.get("java.package"));
    stubs.put("$name$", pageValues.get("dao.name"));

    return SourceUtils.createSourceClass(projectPath, pageValues
        .get("dao.name")
        + "Impl", stubs, "DaoClass");
  }

  public void createDaoInterface(
      Map<String, String> pageValues)
      throws IOException {
    Map<String, String> stubs = new HashMap<String, String>(2);
    stubs.put("$package$", pageValues.get("java.package"));
    stubs.put("$name$", pageValues.get("dao.name"));

    SourceUtils.createSourceClass(projectPath, pageValues.get("dao.name"),
        stubs, "DaoInterface");
  }

  public void updateDeploymentDescriptor(
      Map<String, String> pageValues)
      throws IOException {
    File ddFile = new File(projectPath + File.separatorChar + "conf"
        + File.separatorChar + "web.xml");
    Document doc = WizardUtils.readXmlDocument(ddFile, false);

    if (doc == null) {
      throw new IOException("Unable to read the 'web.xml'");
    }

    doc.getRootElement()
        .add(new Namespace("ns", "http://java.sun.com/xml/ns/j2ee"));

    if (doc.selectSingleNode(
        "//ns:resource-ref[ns:res-ref-name[text()='" +
            pageValues.get("datasource")
            + "']]") == null) {
      Element ref = doc.getRootElement().addElement("resource-ref");
      ref.addElement("res-ref-name").addText(pageValues.get("datasource"));
      ref.addElement("res-type").addText("javax.sql.DataSource");
      ref.addElement("res-auth").addText("Container");

      WizardUtils.outputXmlDocument(ddFile, doc);
    }

    ddFile = new File(projectPath + File.separatorChar + "conf"
        + File.separatorChar + "context.xml");
    doc = WizardUtils.readXmlDocument(ddFile, false);

    if (doc == null) {
      throw new IOException("Unable to read the 'context.xml'");
    }

    if (doc.selectSingleNode(
        "//ResourceLink[@name='" + pageValues.get("datasource") + "']") == null)
    {
      Element ref = doc.getRootElement().addElement("ResourceLink");
      ref.addAttribute("name", pageValues.get("datasource"));
      ref.addAttribute("global", pageValues.get("datasource"));
      ref.addAttribute("type", "javax.sql.DataSource");

      WizardUtils.outputXmlDocument(ddFile, doc);
    }
  }
}
