/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core.handler;

import midstream.core.util.SourceUtils;
import org.dom4j.Document;
import org.dom4j.Element;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class ScheduledServiceHandler extends BaseHandler {

  public static final String[] LIBRARIES = {"cglib:cglib-nodep"};

  public ScheduledServiceHandler(String projectPath) {
    super(projectPath);
  }

  public String createScheduledServiceClass(
      Map<String, String> pageValues)
      throws IOException {
    Map<String, String> stubs = new HashMap<String, String>(2);
    stubs.put("$package$", pageValues.get("java.package"));
    stubs.put("$name$", pageValues.get("service.name"));

    return SourceUtils.createSourceClass(projectPath, pageValues
        .get("service.name"), stubs, "ScheduledServiceClass");
  }

  public void createScheduledServiceClassTest(
      Map<String, String> pageValues)
      throws IOException {
    Map<String, String> stubs = new HashMap<String, String>(2);
    stubs.put("$package$", pageValues.get("java.package"));
    stubs.put("$name$", pageValues.get("service.name"));

    SourceUtils.createSourceClassTest(projectPath, pageValues
        .get("service.name"), stubs, "ScheduledServiceClass");
  }

  public void createServiceContextEntry(
      Map<String, String> pageValues)
      throws IOException {
    Document doc = readApplicationContext();

    if (doc == null) {
      throw new IOException("Unable to read the application's context file");
    }

    Element root = doc.getRootElement();

    if (doc.selectSingleNode("//ns:bean[@id='" + pageValues.get("service.name")
        + "Target" + "']") == null) {
      root.addElement("bean").addAttribute("id",
          pageValues.get("service.name") + "Target")
          .addAttribute(
              "class",
              pageValues.get("java.package") + "."
                  + pageValues.get("service.name"));
    }

    if (doc.selectSingleNode("//ns:bean[@id='" + pageValues.get("service.name")
        + "ScheduledTask']") == null) {
      Element bean = root.addElement("bean").addAttribute("id",
          pageValues.get("service.name") + "ScheduledTask").addAttribute(
          "class", "org.springframework.scheduling.timer.ScheduledTimerTask");
      bean.addElement("property").addAttribute("name", "delay").addAttribute(
          "value", pageValues.get("delay"));
      bean.addElement("property").addAttribute("name", "period").addAttribute(
          "value", pageValues.get("period"));
      bean.addElement("property").addAttribute("name", "timerTask")
          .addAttribute("ref", pageValues.get("service.name"));
    }

    if (doc.selectSingleNode("//ns:bean[@id='" + pageValues.get("service.name")
        + "']") == null) {
      Element bean = root.addElement("bean");
      bean
          .addAttribute("id", pageValues.get("service.name"))
          .addAttribute("class",
              "midstream.framework.scheduling.timer.MethodInvokingTimerTaskFactoryBean")
          .addElement("property").addAttribute("name", "targetObject")
          .addAttribute("ref", pageValues.get("service.name") + "Target");
      bean.addElement("property").addAttribute("name", "targetMethod")
          .addAttribute("value", "execute");
    }

    if (doc.selectSingleNode("//ns:bean[@id='timerFactory']") == null) {
      root.addElement("bean").addAttribute("id", "timerFactory").addAttribute(
          "class", "org.springframework.scheduling.timer.TimerFactoryBean")
          .addElement("property").addAttribute("name", "scheduledTimerTasks")
          .addElement("list").addElement("ref").addAttribute("bean",
          pageValues.get("service.name") + "ScheduledTask");
    }

    outputApplicationContext(doc);
  }
}
