/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core.handler;

import midstream.core.util.SourceUtils;
import org.dom4j.Document;
import org.dom4j.Element;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class SocketClientHandler extends BaseHandler {

  public SocketClientHandler(String projectPath) {
    super(projectPath);
  }

  public String createClientClass(
      Map<String, String> pageValues)
      throws IOException {
    Map<String, String> stubs = new HashMap<String, String>(3);
    stubs.put("$package$", pageValues.get("java.package"));
    stubs.put("$name$", pageValues.get("client.name"));

    return SourceUtils.createSourceClass(projectPath, pageValues
        .get("client.name"), stubs, "SocketClientClass");
  }

  public void createClientClassTest(
      Map<String, String> pageValues)
      throws IOException {
    Map<String, String> stubs = new HashMap<String, String>(3);
    stubs.put("$package$", pageValues.get("java.package"));
    stubs.put("$name$", pageValues.get("client.name"));

    SourceUtils.createSourceClassTest(projectPath, pageValues
        .get("client.name"), stubs, "SocketClientClass");
  }

  public void createContextEntry(
      Map<String, String> pageValues)
      throws IOException {
    Document doc = readApplicationContext();

    if (doc == null) {
      throw new IOException("Unable to read the application's context file");
    }

    Element root = doc.getRootElement();

    if (doc.selectSingleNode("//ns:bean[@id='" + pageValues.get("client.name")
        + "Client']") == null) {
      Element bean = root.addElement("bean").addAttribute("id",
          pageValues.get("client.name") + "Client").addAttribute("class",
          "midstream.framework.remoting.client.SocketClientImpl");
      bean.addElement("property").addAttribute("name", "host").addAttribute(
          "value", pageValues.get("host"));
      bean.addElement("property").addAttribute("name", "port").addAttribute(
          "value", pageValues.get("port"));
      bean.addElement("property").addAttribute("name", "connectionTimeout")
          .addAttribute("value", pageValues.get("connection.timeout"));
      bean.addElement("property").addAttribute("name", "readTimeout")
          .addAttribute("value", pageValues.get("read.timeout"));

      if ("true".equals(pageValues.get("secure"))) {
        bean.addElement("property").addAttribute("name", "secure")
            .addAttribute("value", "true");
        bean.addElement("property").addAttribute("name", "keyStoreLocation")
            .addAttribute("value", pageValues.get("keystore.location"));
        bean.addElement("property").addAttribute("name", "keyStoreType")
            .addAttribute("value", pageValues.get("keystore.type"));
        bean.addElement("property").addAttribute("name", "keyAlgorithm")
            .addAttribute("value", pageValues.get("key.algorithm"));
        bean.addElement("property").addAttribute("name", "secureProtocol")
            .addAttribute("value", pageValues.get("secure.protocol"));
      }
    }

    if (doc.selectSingleNode("//ns:bean[@id='" + pageValues.get("java.package")
        + "." + pageValues.get("client.name") + "']") == null) {
      Element bean = root.addElement("bean");
      bean.addAttribute("id",
          pageValues.get("java.package") + "." + pageValues.get("client.name"))
          .addAttribute(
              "class",
              pageValues.get("java.package") + "."
                  + pageValues.get("client.name"));
      bean.addElement("property").addAttribute("name", "socketClient")
          .addAttribute("ref", pageValues.get("client.name") + "Client");
    }

    outputApplicationContext(doc);
  }
}
