/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core.util;

import java.io.File;
import java.io.IOException;
import java.util.Map;

/**
 * @author Juan Hall
 */
public final class SourceUtils {

  private SourceUtils() {
  }

  public static String createSourceClass(String projectPath, String sourceName,
                                         Map<String, String> replacementValues,
                                         String templatePrefix)
      throws IOException {
    String srcPath = "src" + File.separatorChar
        + replacementValues.get("$package$").replace('.', File.separatorChar);
    File dir = new File(projectPath + File.separatorChar + srcPath);
    dir.mkdirs();
    String contents = WizardUtils.readStream(SourceUtils.class.getClassLoader()
        .getResourceAsStream(
        "midstream/core/templates/components/"
            + templatePrefix + ".template"));

    for (Map.Entry<String, String> entry : replacementValues.entrySet()) {
      contents = contents.replaceAll(entry.getKey()
          .replaceAll("\\$", "\\\\\\$"), entry.getValue());
    }

    WizardUtils.writeFile(dir.getAbsolutePath() + File.separatorChar
        + sourceName + ".java", contents, false);

    return srcPath + File.separatorChar + sourceName + ".java";
  }

  public static void createSourceClassTest(String projectPath,
                                           String sourceName,
                                           Map<String, String> replacementValues,
                                           String templatePrefix)
      throws IOException {
    File dir = new File(projectPath + File.separatorChar + "test"
        + File.separatorChar + "src" + File.separatorChar
        + replacementValues.get("$package$").replace('.', File.separatorChar));
    dir.mkdirs();
    String contents = WizardUtils.readStream(SourceUtils.class.getClassLoader()
        .getResourceAsStream(
        "midstream/core/templates/components/"
            + templatePrefix + "Test.template"));

    for (Map.Entry<String, String> entry : replacementValues.entrySet()) {
      contents = contents.replaceAll(entry.getKey()
          .replaceAll("\\$", "\\\\\\$"), entry.getValue());
    }

    WizardUtils.writeFile(dir.getAbsolutePath() + File.separatorChar
        + sourceName + "Test.java", contents, false);
  }
}
