/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core.views;

import midstream.core.util.WizardUtils;
import org.dom4j.Attribute;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.tigris.gef.base.CmdSaveGIF;
import org.tigris.gef.base.Editor;
import org.tigris.gef.base.Globals;
import org.tigris.gef.base.Layer;
import org.tigris.gef.base.LayerGrid;
import org.tigris.gef.base.LayerPerspective;
import org.tigris.gef.graph.GraphModel;
import org.tigris.gef.graph.presentation.DefaultGraphModel;
import org.tigris.gef.graph.presentation.JGraph;
import org.tigris.gef.graph.presentation.NetEdge;
import org.tigris.gef.graph.presentation.NetNode;
import org.tigris.gef.graph.presentation.NetPort;
import org.tigris.gef.presentation.ArrowHeadTriangle;
import org.tigris.gef.presentation.Fig;
import org.tigris.gef.presentation.FigEdge;
import org.tigris.gef.presentation.FigEdgeLine;
import org.tigris.gef.presentation.FigGroup;
import org.tigris.gef.presentation.FigImage;
import org.tigris.gef.presentation.FigNode;
import org.tigris.gef.presentation.FigRect;
import org.tigris.gef.presentation.FigText;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JPanel;
import java.awt.Color;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Vector;

/**
 * @author Juan Hall
 */
public final class ContextViewRenderer {

  private ContextViewRenderer() {
  }

  private static String projectRootPath = "";

  public static void paintContext(final String projectPath, final JPanel panel,
                                  ContextViewCallback viewCallback) {
    projectRootPath = projectPath;

    Document doc = parseFile("applicationContext.xml");
    List<?> nodes = doc.selectNodes("//ns:bean[@id or @name]");
    List<List<String>> levels = new ArrayList<List<String>>(5);
    List<Object> toRemove = new ArrayList<Object>(5);

    for (int level = 0; !nodes.isEmpty(); ++level) {
      toRemove.clear();
      for (Object node : nodes) {
        String bean =
            ((Element) node).attributeValue("name") == null ? ((Element) node)
                .attributeValue("id")
                : ((Element) node).attributeValue("name");
        List references = getReferences(doc, bean);
        if ((level == 0 && references.isEmpty())
            || isIn(references, levels, level)) {
          if (levels.size() <= level) {
            levels.add(new ArrayList<String>(5));
          }
          levels.get(level).add(bean);

          toRemove.add(node);
        }
      }
      nodes.removeAll(toRemove);
    }

    createGraphPanel(doc, levels, panel, viewCallback);
  }

  public static void zoom(String value) {
    Editor ed = Globals.curEditor();

    if (ed != null && value != null) {
      String scale = value.replaceFirst("%", "");
      int convertedScale;

      try {
        convertedScale = Integer.parseInt(scale);
      } catch (NumberFormatException e) {
        convertedScale = 1;
      }

      convertedScale = (convertedScale <= 0) ? 1 : convertedScale;

      ed.setScale((double) convertedScale / 100);
    }
  }

  public static boolean saveViewCoords(String projectPath) {
    Editor ed = Globals.curEditor();

    Document document = DocumentHelper.createDocument();
    Element root = document.addElement("nodes");

    for (Object node : ed.getGraphModel().getNodes()) {
      if (node instanceof MyNode) {
        Fig fig = ((MyNode) node).getFigNode();
        root.addElement("node").addAttribute("id", ((MyNode) node).getId())
            .addAttribute("x", Integer.toString(fig.getX())).addAttribute("y",
            Integer.toString(fig.getY()));
      }
    }

    WizardUtils.outputXmlDocument(new File(projectPath + File.separatorChar
        + "conf" + File.separatorChar + "context_view.xml"), document);

    return true;
  }

  private static Map<String, int[]> getNodeCoords() {
    Document doc = parseFile("context_view.xml");

    if (doc == null) {
      return null;
    }

    Map<String, int[]> nodes = new HashMap<String, int[]>(5);

    for (Object xmlNode : doc.getRootElement().selectNodes("//node")) {
      Element curNode = (Element) xmlNode;

      nodes.put(curNode.attributeValue("id"), new int[]{
          Integer.valueOf(curNode.attributeValue("x")),
          Integer.valueOf(curNode.attributeValue("y"))});
    }

    return nodes;
  }

  public static void saveAsGif(String fileName) {
    CmdSaveGIF cmd = new CmdSaveGIF();
    FileOutputStream os = null;

    try {
      os = new FileOutputStream(fileName);
      cmd.setStream(os);

      cmd.doIt();
    } catch (IOException e) {
      e.printStackTrace();
    } finally {
      if (os != null) {
        try {
          os.close();
        } catch (IOException e) {
          e.printStackTrace();
        }
      }
    }
  }

  private static Document parseFile(final String fileName) {
    Document doc = null;

    try {
      doc = WizardUtils
          .readXmlDocument(new File(projectRootPath + File.separatorChar
              + "conf" + File.separatorChar + fileName), false);

      if (doc != null) {
        doc.getRootElement().add(
            new Namespace("ns", "http://www.springframework.org/schema/beans"));
      }
    } catch (Exception e) {
      e.printStackTrace();
    }

    return doc;
  }

  private static boolean isIn(List references, List<List<String>> list,
                              int level) {
    if (level == 0 || list.size() <= level - 1) {
      return false;
    }

    List nodes = list.get(level - 1);

    for (Object node : references) {
      String bean =
          ((Element) node).attributeValue("name") == null ? ((Element) node)
              .attributeValue("id")
              : ((Element) node).attributeValue("name");

      if (nodes.contains(bean)) {
        return true;
      }
    }

    return false;
  }

  private static List getReferences(Document doc, String bean) {
    return doc.selectNodes("//ns:bean[ns:property/@value='" + bean
        + "' or ns:property/@ref='" + bean + "']");
  }

  private static void createGraphPanel(Document doc, List<List<String>> levels,
                                       JPanel panel,
                                       ContextViewCallback viewCallback) {
    JGraph graph;

    if (panel.getComponentCount() < 1) {
      graph = new JGraph(new Editor(new DefaultGraphModel(), null) {
        private static final long serialVersionUID = 1L;

        @Override
        protected void defineLayers(GraphModel model, Layer layer) {
          _layerManager.addLayer(new LayerGrid(Color.WHITE, Color.WHITE, 16,
              false));
          _layerManager.addLayer(new LayerPerspective("untitled", model));
        }
      });
      panel.add(graph);
    } else {
      graph = (JGraph) panel.getComponent(0);
      clearGraph(graph);
    }

    LayerPerspective layer = (LayerPerspective) graph.getEditor()
        .getLayerManager().getActiveLayer();
    layer.allowNetClass(MyNode.class);
    layer.allowNetClass(MyEdge.class);

    int x = 10;
    int y = 10;
    int maxWidth = 0;
    int maxHeight = 0;
    List<Integer> widths = new ArrayList<Integer>(5);
    Map<String, MyNode> nodes = new HashMap<String, MyNode>(5);
    Map<String, int[]> savedNodes = getNodeCoords();

    for (List<String> level : levels) {
      for (final String bean : level) {
        if (savedNodes != null && savedNodes.containsKey(bean)) {
          int[] coords = savedNodes.get(bean);
          x = coords[0];
          y = coords[1];
        }

        MyNode netNode = new MyNode((Element) doc
            .selectSingleNode("//ns:bean[@id='" + bean + "' or @name='" + bean
                + "']"), x, y, viewCallback);
        NetPort np = new NetPort(netNode);
        netNode.addPort(np);
        netNode.getFigNode().setOwner(np);
        netNode.getPortTop().setOwner(np);

        np = new NetPort(netNode);
        netNode.addPort(np);
        netNode.getPortBottom().setOwner(np);
        nodes.put(bean, netNode);

        ((DefaultGraphModel) graph.getGraphModel()).addNode(netNode);

        x += 10 + netNode.getFigNode().getWidth();

        if (netNode.getFigNode().getHeight() > maxHeight) {
          maxHeight = netNode.getFigNode().getHeight();
        }
      }

      if (x > maxWidth) {
        maxWidth = x;
      }

      widths.add(x);

      x = 10;
      y += 70 + maxHeight;
    }

    createEdges(doc, levels, graph, maxWidth, widths, nodes);
  }

  private static void clearGraph(JGraph graph) {
    ((DefaultGraphModel) graph.getGraphModel()).removeAll();
    graph.getEditor().getLayerManager().removeAll();
    graph.deselectAll();
  }

  private static void createEdges(Document doc, List<List<String>> levels,
                                  JGraph graph, int maxWidth,
                                  List<Integer> widths,
                                  Map<String, MyNode> nodes) {
    int index = 0;
    boolean hasSavedCoords = getNodeCoords() != null;
    for (List<String> level : levels) {
      for (final String bean : level) {
        if (!hasSavedCoords) {
          nodes.get(bean).getFigNode().setX(
              nodes.get(bean).getFigNode().getX()
                  + (maxWidth - widths.get(index)) / 2);
        }

        List refs = getReferences(doc, bean);
        MyNode src = nodes.get(bean);

        for (Object ref : refs) {
          String beanRef =
              ((Element) ref).attributeValue("name") == null ? ((Element) ref)
                  .attributeValue("id")
                  : ((Element) ref).attributeValue("name");
          MyNode dest = nodes.get(beanRef);
          MyEdge fe = new MyEdge();
          fe.setSourcePort(src.getPort(0));
          fe.setDestPort(dest.getPort(1));

          ((DefaultGraphModel) graph.getGraphModel()).addEdge(fe);
        }
      }

      index++;
    }
  }
}

/**
 * @author Juan Hall
 */
class MyEdge extends NetEdge {
  private static final long serialVersionUID = 1L;

  private FigEdge edge = new FigEdgeLine() {
    private static final long serialVersionUID = 1L;

    @Override
    public Vector getPopUpActions(MouseEvent arg0) {
      return new Vector();
    }
  };

  public MyEdge() {
    edge.setSourceArrowHead(new ArrowHeadTriangle());
  }

  public String getId() {
    return toString();
  }

  @Override
  public FigEdge makePresentation(Layer lay) {
    return edge;
  }
}

/**
 * @author Juan Hall
 */
class ImageText extends FigGroup {

  private static final long serialVersionUID = 1L;

  private static final Color NODE_COLOR = new Color(255, 255, 210);

  private String text;

  public ImageText(int x, int y, URL imageUrl, String text, boolean bold,
                   int fontSize) {
    FigRect rect = new FigRect(x, y, 0, 0);
    rect.setFillColor(NODE_COLOR);
    addFig(rect);

    FigImage image = new FigImage(x + 2, y, new ImageIcon(imageUrl).getImage());
    addFig(image);

    FigText figText = new FigText(image.getX() + image.getWidth() + 2, y, 0, 0);
    figText.setText(text);
    figText.setBold(bold);
    figText.setFontSize(fontSize);
    figText.setLineWidth(0);
    figText.setFillColor(NODE_COLOR);
    figText.setJustification(FigText.JUSTIFY_LEFT);

    addFig(figText);

    rect.setSize(this.getWidth(), this.getHeight() + 2);

    image.setCenter(new Point(image.getCenter().x, rect.getCenter().y));
    figText.setCenter(new Point(figText.getCenter().x, rect.getCenter().y));

    setText(text);
  }

  public void setFigWidth(int width) {
    this.getFigAt(0).setWidth(width);
  }

  public String getText() {
    return text;
  }

  public void setText(String text) {
    this.text = text;
  }
}

/**
 * @author Juan Hall
 */
class MyNode extends NetNode {

  private static final long serialVersionUID = 1L;

  private ImageText titleGroup;

  private ContextViewCallback viewCallback;

  public MyNode(Element bean, int x, int y, ContextViewCallback viewCallback) {
    FigRect shadow = new FigRect(x, y, 0, 0);
    shadow.setFillColor(Color.GRAY);
    shadow.setLineWidth(0);
    figNode.addFig(shadow);

    createNodeTitle(bean, x, y);

    createPropertyFigs(bean, x, y);

    for (Object fig : figNode.getFigs()) {
      if (fig instanceof ImageText) {
        ((ImageText) fig).setFigWidth(figNode.getWidth());
      }
    }

    portTop = new FigRect(figNode.getX() + (figNode.getWidth() - 1) / 2,
        figNode.getY(), 0, 0);
    figNode.addFig(portTop);

    portBottom = new FigRect(figNode.getX() + (figNode.getWidth() - 1) / 2,
        figNode.getY() + figNode.getHeight() + 2, 0, 0);
    figNode.addFig(portBottom);

    shadow.setBounds(figNode.getX() + 3, figNode.getY() + 3,
        figNode.getWidth(), figNode.getHeight());

    this.viewCallback = viewCallback;
  }

  private void createNodeTitle(Element bean, int x, int y) {
    titleGroup = new ImageText(x, y, this.getClass().getClassLoader()
        .getResource(
        "midstream/core/resources/icons/beanIcon.gif"), bean
        .attributeValue("name") == null ? bean.attributeValue("id") : bean
        .attributeValue("name"), true, 14);
    figNode.addFig(titleGroup);
  }

  private void createPropertyFigs(Element bean, int x, int y) {
    int height = y + this.titleGroup.getHeight();

    for (Object attr : bean.selectNodes("./ns:property/@name")) {
      final ImageText imageText = new ImageText(x, height, this.getClass()
          .getClassLoader().getResource(
          "midstream/core/resources/icons/beanProperty.gif"),
          ((Attribute) attr).getText(), false, 12);
      figNode.addFig(imageText);
      height += imageText.getHeight() - 1;
    }
  }

  private FigRect portBottom;

  public Fig getPortBottom() {
    return portBottom;
  }

  private FigRect portTop;

  public Fig getPortTop() {
    return portTop;
  }

  @Override
  public void initialize(Hashtable arg0) {
  }

  private FigNode figNode = new FigNode() {
    private static final long serialVersionUID = 1L;

    @Override
    public Vector getPopUpActions(MouseEvent mEvent) {
      Vector<JMenu> actions = new Vector<JMenu>();
      JMenu popupMenu = new JMenu("Navigate");

      popupMenu.setMnemonic('N');
      popupMenu.add(new AbstractAction("Jump to Class") {
        private static final long serialVersionUID = 1L;

        public void actionPerformed(ActionEvent aEvent) {
          viewCallback.openSource(titleGroup.getText());
        }
      });
      popupMenu.add(new AbstractAction("Jump to Configuration") {
        private static final long serialVersionUID = 1L;

        public void actionPerformed(ActionEvent aEvent) {
          viewCallback.openConfiguration(titleGroup.getText());
        }
      });

      actions.addElement(popupMenu);

      popupMenu = new JMenu("Source");

      popupMenu.setMnemonic('S');
      popupMenu.add(new AbstractAction("Add Property") {
        private static final long serialVersionUID = 1L;

        public void actionPerformed(ActionEvent aEvent) {
          viewCallback.addProperty(titleGroup.getText());
        }
      });

      actions.addElement(popupMenu);

      return actions;
    }
  };

  public FigNode getFigNode() {
    return figNode;
  }

  @Override
  public FigNode makePresentation(Layer arg0) {
    return figNode;
  }

  @Override
  public String getId() {
    return titleGroup.getText();
  }
}