/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core.handler;

import junit.framework.TestCase;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.springframework.util.FileCopyUtils;

import java.io.File;

/**
 * @author Juan Hall
 */
public class ConfigurationBeanHandlerTest extends TestCase {

  private static final String TMP_DIR =
      "test/tmp/" + ConfigurationBeanHandlerTest.class.getSimpleName();

  @BeforeClass
  @Override
  protected void setUp() throws Exception {
    super.setUp();

    new File(TMP_DIR).mkdirs();
    FileCopyUtils.copy(
        new File(
            "src/midstream/core/templates/project/build.xml"),
        new File(TMP_DIR + "/build.xml"));
  }

  @AfterClass
  @Override
  protected void tearDown() throws Exception {
    new File(TMP_DIR + "/build.xml").delete();
    new File(TMP_DIR).delete();

    super.tearDown();
  }

  @Test
  public void testUpdateBuildFile() throws Exception {
    new ConfigurationBeanHandler(TMP_DIR).updateBuildFile();

    SAXReader reader = new SAXReader();
    Document doc = reader.read(TMP_DIR + "/build.xml");
    Node node = doc.selectSingleNode("//webinf/@includes");

    assertTrue(node.getText().contains(", configuration.xml.*"));
  }

  @Test
  public void testAddConfigurationControllerBean() {
    Element root = DocumentHelper.createElement("beans");

    ConfigurationBeanHandler.addConfigurationControllerBean(root);

    assertNotNull(root.selectSingleNode(
        "./bean[@id='paramResolver' and @class='org.springframework.web.servlet.mvc.multiaction.ParameterMethodNameResolver']/property[@name='paramName' and @value='method']"));
    assertNotNull(root.selectSingleNode(
        "./bean[@name='/Configuration' and @class='midstream.framework.configuration.ConfigurationController' and property[@name='configuration' and @ref='configuration'] and property[@name='methodNameResolver' and @ref='paramResolver']]"));
  }
}
