/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.core.handler;

import junit.framework.TestCase;
import org.dom4j.Document;
import org.dom4j.Namespace;
import org.dom4j.io.SAXReader;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.springframework.util.FileCopyUtils;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class DaoHandlerTest extends TestCase {

  private static final String TMP_DIR =
      "test/tmp/" + DaoHandlerTest.class.getSimpleName();

  @BeforeClass
  @Override
  protected void setUp() throws Exception {
    super.setUp();

    new File(TMP_DIR + "/conf").mkdirs();
    FileCopyUtils.copy(
        new File(
            "src/midstream/core/templates/project/web.xml"),
        new File(TMP_DIR + "/conf/web.xml"));
    FileCopyUtils.copy(
        new File(
            "src/midstream/core/templates/project/context.xml"),
        new File(TMP_DIR + "/conf/context.xml"));
  }

  @AfterClass
  @Override
  protected void tearDown() throws Exception {
    new File(TMP_DIR + "/conf/web.xml").delete();
    new File(TMP_DIR + "/conf/context.xml").delete();
    new File(TMP_DIR + "/conf").delete();
    new File(TMP_DIR).delete();

    super.tearDown();
  }

  @Test
  public void testUpdateDeploymentDescriptor() throws Exception {
    Map<String, String> pageValues = new HashMap<String, String>(5);
    pageValues.put("datasource", "MyDS");

    new DaoHandler(TMP_DIR).updateDeploymentDescriptor(pageValues);

    SAXReader reader = new SAXReader();
    Document doc = reader.read(TMP_DIR + "/conf/web.xml");
    doc.getRootElement()
        .add(new Namespace("ns", "http://java.sun.com/xml/ns/j2ee"));

    assertNotNull(doc.selectSingleNode(
        "//ns:resource-ref[ns:res-ref-name[text()='MyDS'] and ns:res-type[text()='javax.sql.DataSource'] and ns:res-auth[text()='Container']]"));

    doc = reader.read(TMP_DIR + "/conf/context.xml");

    assertNotNull(doc.selectSingleNode(
        "//ResourceLink[@name='MyDS' and @global='MyDS' and @type='javax.sql.DataSource']"));
  }
}
