/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import midstream.core.handler.AddBeanPropertyHandler;
import midstream.util.IdeUtils;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class AddBeanPropertyDialog extends JDialog {
  private JPanel contentPane;
  private JButton buttonOK;
  private JButton buttonCancel;
  private JTextField selectedBeanText;
  private JTextField propertyNameText;
  private JCheckBox addToBeanSource;
  private JTabbedPane propertyTypeFolder;
  private JComboBox referenceBeanCombo;
  private JTextField fqnReferenceTypeText;
  private JList referenceTypeList;
  private JTextField valueText;
  private JComboBox valueTypeCombo;
  private JComboBox collectionTypeCombo;
  private JTable collectionTable;
  private boolean ok = false;
  private Map<String, Object> pageValues = new HashMap<String, Object>(5);
  private List<String> classes = new ArrayList<String>(5);
  private Thread classesThread;
  private String selectedBeanName;
  private String projectPath;
  private JLabel propertyNameLabel;
  private JLabel selectedBeanLabel;

  public AddBeanPropertyDialog(JFrame parent, final String projectPath,
                               String beanName) {
    super(parent, "Add Bean Property", true);
    this.selectedBeanName = beanName;
    this.projectPath = projectPath;

    setContentPane(contentPane);
    getRootPane().setDefaultButton(buttonOK);

    buttonOK.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onOK();
      }
    });

    buttonCancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    });

// call onCancel() when cross is clicked
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        onCancel();
      }
    });

// call onCancel() on ESCAPE
    contentPane.registerKeyboardAction(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
        JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

    if (beanName == null) {
      this.selectedBeanLabel.setVisible(false);
      this.selectedBeanText.setVisible(false);
      this.propertyNameLabel.setVisible(false);
      this.propertyNameText.setVisible(false);
      this.addToBeanSource.setVisible(false);
    } else {
      this.selectedBeanText.setText(beanName);
    }

    configureReferenceTab();

    configureValueTab();

    configureCollectionTab();

    IdeUtils.packAndCenter(this);

    classesThread = new Thread() {
      @Override
      public void run() {
        classes = new AddBeanPropertyHandler(projectPath).buildClassNameList();
      }
    };

    classesThread.start();
  }

  private void configureReferenceTab() {
    this.propertyTypeFolder.setTitleAt(0, "Reference");

    populateReferenceBeanCombo();

    this.fqnReferenceTypeText.addFocusListener(new FocusAdapter() {
      public void focusGained(FocusEvent focusEvent) {
        joinClassThread();
      }
    });
    this.fqnReferenceTypeText.addKeyListener(new KeyAdapter() {
      public void keyReleased(KeyEvent keyEvent) {
        referenceTypeList.setListData(
            new AddBeanPropertyHandler(projectPath).getMatchingItems(
                fqnReferenceTypeText.getText(), classes));
      }
    });
    this.referenceTypeList
        .addListSelectionListener(new ListSelectionListener() {
          public void valueChanged(ListSelectionEvent listSelectionEvent) {
            fqnReferenceTypeText
                .setText(referenceTypeList.getSelectedValue().toString());
          }
        });
  }

  private void configureValueTab() {
    this.propertyTypeFolder.setTitleAt(1, "Value");

    for (String type : new String[]{"String", "char", "int",
        "long", "float", "double", "boolean"}) {
      this.valueTypeCombo.addItem(type);
    }
  }

  private void configureCollectionTab() {
    this.propertyTypeFolder.setTitleAt(2, "Collection");

    for (String type : new String[]{"Properties", "List",
        "Map", "Set"}) {
      this.collectionTypeCombo.addItem(type);
    }

    DefaultCellEditor cellEditor = new DefaultCellEditor(new JTextField());
    cellEditor.setClickCountToStart(1);

    DefaultTableModel model = new DefaultTableModel() {
      public boolean isCellEditable(int row, int col) {
        return col > 0;
      }

      public Class<?> getColumnClass(int i) {
        return getValueAt(0, i).getClass();
      }
    };

    for (String name : new String[]{"Ref", "Key", "Value"}) {
      model.addColumn(name);
    }

    for (int i = 0; i < 20; ++i) {
      model.addRow(new Object[]{Boolean.FALSE, "", ""});
    }

    this.collectionTable.setModel(model);
    this.collectionTable.getColumnModel().getColumn(0).setMaxWidth(25);
    this.collectionTable.getColumnModel().getColumn(1)
        .setCellEditor(cellEditor);
    this.collectionTable.getColumnModel().getColumn(2)
        .setCellEditor(cellEditor);
  }

  private void populateReferenceBeanCombo() {
    List<String> names =
        new AddBeanPropertyHandler(this.projectPath).getContextBeanNames();

    this.referenceBeanCombo.removeAllItems();

    for (String name : names) {
      this.referenceBeanCombo.addItem(name);
    }
  }

  private void joinClassThread() {
    if (classesThread != null && classesThread.isAlive()) {
      try {
        classesThread.join(10000);
      } catch (InterruptedException e1) {
        e1.printStackTrace();
      }
    }
  }

  public boolean isOk() {
    return ok;
  }

  private void onOK() {
    joinClassThread();

    ok = true;
    if (validateInput()) {
      setPageValues();
      dispose();
    }
  }

  private void onCancel() {
    joinClassThread();

    ok = false;
    dispose();
  }

  private boolean validateInput() {
    if (this.selectedBeanName != null &&
        "".equals(this.propertyNameText.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A property name must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return false;
    } else if (this.selectedBeanName != null &&
        Character.isUpperCase(this.propertyNameText.getText().trim().charAt(0)))
    {
      JOptionPane.showMessageDialog(this,
          "Property names must begin with a lowercase character.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return false;
    }

    switch (this.propertyTypeFolder.getSelectedIndex()) {
      case AddBeanPropertyHandler.REFERENCE:
        if ("".equals(this.fqnReferenceTypeText.getText().trim())) {
          JOptionPane.showMessageDialog(this,
              "A full-qualified type must be entered.", "Error",
              JOptionPane.ERROR_MESSAGE);
          return false;
        }
        break;
      case AddBeanPropertyHandler.VALUE:
        if ("".equals(this.valueText.getText().trim())) {
          JOptionPane.showMessageDialog(this, "A value must be entered.",
              "Error", JOptionPane.ERROR_MESSAGE);
          return false;
        }
        break;
      case AddBeanPropertyHandler.COLLECTION:
        if (IdeUtils.isTableEmpty(this.collectionTable, 2)) {
          JOptionPane.showMessageDialog(this,
              "At least one entry must be made.", "Error",
              JOptionPane.ERROR_MESSAGE);
          return false;
        }
        break;
    }

    return true;
  }

  private void setPageValues() {
    if (this.selectedBeanName != null) {
      this.pageValues
          .put("selected.bean", this.selectedBeanText.getText().trim());
      this.pageValues
          .put("property.name", this.propertyNameText.getText().trim());
      this.pageValues
          .put("add.getter.setter", this.addToBeanSource.isSelected());
    }

    this.pageValues
        .put("property.tab.index", this.propertyTypeFolder.getSelectedIndex());

    switch (this.propertyTypeFolder.getSelectedIndex()) {
      case AddBeanPropertyHandler.REFERENCE:
        this.pageValues
            .put("reference.bean", this.referenceBeanCombo.getSelectedItem());
        this.pageValues
            .put("fqn.type", this.fqnReferenceTypeText.getText().trim());
        break;
      case AddBeanPropertyHandler.VALUE:
        this.pageValues.put("value", this.valueText.getText().trim());
        this.pageValues
            .put("value.type", this.valueTypeCombo.getSelectedItem());
        break;
      case AddBeanPropertyHandler.COLLECTION:
        List<String[]> collection = new ArrayList<String[]>(5);

        for (int i = 0; i < collectionTable.getRowCount(); ++i) {
          collection
              .add(new String[]{collectionTable.getValueAt(i, 0).toString(),
                  collectionTable.getValueAt(i, 1).toString(),
                  collectionTable.getValueAt(i, 2).toString()});
        }

        this.pageValues.put("collection", collection);
        this.pageValues
            .put("collection.type", this.collectionTypeCombo.getSelectedItem());
        break;
    }
  }

  public Map<String, Object> getPageValues() {
    return this.pageValues;
  }
}
