/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.execution.ui.ConsoleView;
import com.intellij.execution.ui.ConsoleViewContentType;
import com.intellij.openapi.editor.markup.TextAttributes;
import com.intellij.openapi.util.Key;
import midstream.core.ConsoleOperation;

import javax.swing.JComponent;
import javax.swing.JOptionPane;
import java.awt.Color;
import java.awt.Cursor;

/**
 * @author Juan Hall
 */
public class ConsoleOperationImpl implements ConsoleOperation {

  private ConsoleView console;
  private JComponent runnerWindow;
  private static final Key HEADER_OUTPUT = Key.create("HEADER_OUTPUT");
  private static final Key VALUE_OUTPUT = Key.create("VALUE_OUTPUT");
  private Cursor oldCursor;

  static {
    TextAttributes attrs = new TextAttributes();
    attrs.setForegroundColor(new Color(0, 0, 128));
    ConsoleViewContentType viewType =
        new ConsoleViewContentType("HEADER_OUTPUT", attrs) {
        };
    ConsoleViewContentType.registerNewConsoleViewType(HEADER_OUTPUT, viewType);

    attrs = new TextAttributes();
    attrs.setForegroundColor(new Color(128, 0, 0));
    viewType = new ConsoleViewContentType("HEADER_OUTPUT", attrs) {
    };
    ConsoleViewContentType.registerNewConsoleViewType(VALUE_OUTPUT, viewType);
  }

  public ConsoleOperationImpl(ConsoleView console, JComponent window) {
    this.console = console;
    this.runnerWindow = window;

    oldCursor = this.runnerWindow.getCursor();
    this.runnerWindow.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

    if (!console.getComponent().isVisible()) {
      console.getComponent().setVisible(true);
    }

    console.clear();
  }

  public void dispose() {
    this.runnerWindow.setCursor(oldCursor);
  }

  public void printHeader(String message) {
    console.print(message,
        ConsoleViewContentType.getConsoleViewType(HEADER_OUTPUT));
  }

  public void printlnHeader(String message) {
    if (message == null) {
      console.print(System.getProperty("line.separator"),
          ConsoleViewContentType.getConsoleViewType(HEADER_OUTPUT));
    } else {
      console.print(message + System.getProperty("line.separator"),
          ConsoleViewContentType.getConsoleViewType(HEADER_OUTPUT));
    }
  }

  public void printValue(String message) {
    console.print(message,
        ConsoleViewContentType.getConsoleViewType(VALUE_OUTPUT));
  }

  public void printlnValue(String message) {
    if (message == null) {
      console.print(System.getProperty("line.separator"),
          ConsoleViewContentType.getConsoleViewType(VALUE_OUTPUT));
    } else {
      console.print(message + System.getProperty("line.separator"),
          ConsoleViewContentType.getConsoleViewType(VALUE_OUTPUT));
    }
  }

  public void printError(String message) {
    console.print(message, ConsoleViewContentType.ERROR_OUTPUT);
  }

  public void printlnError(String message) {
    if (message == null) {
      console.print(System.getProperty("line.separator"),
          ConsoleViewContentType.ERROR_OUTPUT);
    } else {
      console.print(message + System.getProperty("line.separator"),
          ConsoleViewContentType.ERROR_OUTPUT);
    }
  }

  public void showErrorMessage(String message) {
    JOptionPane.showMessageDialog(null, message, "Error",
        JOptionPane.ERROR_MESSAGE);
  }
}
