/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.diagnostic.Logger;
import midstream.core.handler.AddBeanPropertyHandler;
import midstream.core.util.WizardUtils;
import midstream.core.views.ContextViewCallback;
import midstream.util.IdeUtils;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.Namespace;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import java.io.File;
import java.io.IOException;

/**
 * @author Juan Hall
 */
public class ContextViewCallbackImpl implements ContextViewCallback {

  private ContextViewForm contextView;
  private String projectPath;

  public ContextViewCallbackImpl(ContextViewForm view,
                                 final String projectPath) {
    this.contextView = view;
    this.projectPath = projectPath;
  }

  public void openConfiguration(String string) {
    IdeUtils.openFile(this.contextView.getProject(),
        this.projectPath + File.separatorChar +
            "conf" + File.separatorChar +
            "applicationContext.xml");
  }

  public void openSource(String name) {
    try {
      Document doc = WizardUtils.readXmlDocument(new File(this.projectPath
          + File.separatorChar
          + "conf"
          + File.separatorChar
          + "applicationContext.xml"), false);

      if (doc == null) {
        throw new IOException("Unable to read the application's context file");
      }

      doc.getRootElement().add(
          new Namespace("ns", "http://www.springframework.org/schema/beans"));

      final Element bean = (Element) doc
          .selectSingleNode("//ns:bean[@name='%' or @id='%']"
              .replaceAll("%", name));

      if (bean != null && bean.attributeValue("class") != null) {
        IdeUtils.openFile(contextView.getProject(),
            this.projectPath + File.separatorChar + "src" + File.separatorChar
                + bean.attributeValue("class").replace('.', File.separatorChar)
                + ".java");
      }
    } catch (IOException ex) {
      Logger.getInstance(this.getClass().getName()).error(ex);
    }
  }

  public void addProperty(final String beanName) {
    Runnable runner = new Runnable() {
      public void run() {
        JFrame frame = IdeUtils.suggestParentWindow(contextView.getProject());
        AddBeanPropertyDialog dialog = new AddBeanPropertyDialog(
            frame, projectPath, beanName);
        dialog.setVisible(true);

        if (dialog.isOk()) {
          String fileName;

          try {
            fileName = new AddBeanPropertyHandler(projectPath).addBeanProperty(
                dialog.getPageValues(), beanName);
          } catch (Exception e) {
            JOptionPane.showMessageDialog(frame, e.getMessage(), "Error",
                JOptionPane.ERROR_MESSAGE);
            return;
          }

          if (fileName != null) {
            openSource(beanName);
            IdeUtils.reloadProject();
          }

          contextView.refreshView();
        }
      }
    };

    IdeUtils.rebuildProject(contextView.getProject(), runner);
  }
}
