/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.project.Project;
import midstream.core.views.ContextViewRenderer;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.filechooser.FileFilter;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

/**
 * @author Juan Hall
 */
public class ContextViewForm {
  private JPanel rootComponent;
  private JPanel graphPanel;
  private JPanel toolbarPanel;
  private JButton refreshButton;
  private JButton saveCoordsButton;
  private JButton saveAsImageButton;
  private JComboBox zoomCombo;
  private Project project;
  private String projectPath;

  public ContextViewForm(Project project) {
    this.project = project;
    projectPath = project.getProjectFilePath().substring(0, project
        .getProjectFilePath().indexOf(project.getProjectFile().getName()) - 1);

    createRefreshButton();

    createSaveCoordinatesButton();

    createSaveAsImageButton();

    createZoomCombo();

    createToolBar();

    this.graphPanel.setLayout(new BorderLayout());
    this.graphPanel.setBackground(Color.WHITE);

    refreshView();
  }

  private void createToolBar() {
    JToolBar toolbar = new JToolBar(JToolBar.HORIZONTAL);
    toolbar.setBorderPainted(false);
    toolbar.setFloatable(false);
    toolbar.add(refreshButton);
    toolbar.add(saveCoordsButton);
    toolbar.add(saveAsImageButton);
    toolbar.add(zoomCombo);
    toolbarPanel.setLayout(new FlowLayout());
    toolbarPanel.add(toolbar);
  }

  private void createRefreshButton() {
    refreshButton = new JButton(new ImageIcon(
        getClass().getClassLoader().getResource(
            "midstream/core/resources/icons/refresh.gif")));
    refreshButton.setMargin(new Insets(0, 0, 0, 0));
    refreshButton.setToolTipText("Refresh");
    refreshButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        refreshView();
      }
    });
  }

  private void createSaveCoordinatesButton() {
    saveCoordsButton = new JButton(new ImageIcon(
        getClass().getClassLoader().getResource(
            "midstream/core/resources/icons/save.gif")));
    saveCoordsButton.setMargin(new Insets(0, 0, 0, 0));
    saveCoordsButton.setToolTipText("Save");
    saveCoordsButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        String message = "Unable to save view. See error logging for details.";

        if (ContextViewRenderer.saveViewCoords(projectPath)) {
          message = "Save complete!";
        }

        JOptionPane.showMessageDialog(getRootComponent(), message, "Save",
            JOptionPane.INFORMATION_MESSAGE);
      }
    });
  }

  private void createSaveAsImageButton() {
    saveAsImageButton = new JButton(new ImageIcon(
        getClass().getClassLoader().getResource(
            "midstream/core/resources/icons/saveAsGif.gif")));
    saveAsImageButton.setMargin(new Insets(0, 0, 0, 0));
    saveAsImageButton.setToolTipText("Save As GIF");
    saveAsImageButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        JFileChooser saveDialog = new JFileChooser();
        saveDialog.setDialogTitle("Save As GIF");
        saveDialog.setFileFilter(new FileFilter() {
          public boolean accept(File file) {
            return file.isDirectory() || file.getName().endsWith(".gif");
          }

          public String getDescription() {
            return "*.gif";
          }
        });

        if (saveDialog.showSaveDialog(getRootComponent()) == JFileChooser
            .APPROVE_OPTION && saveDialog.getSelectedFile() != null) {
          String fileName =
              saveDialog.getSelectedFile().getAbsolutePath().trim();
          if (!"".equals(fileName)) {
            ContextViewRenderer
                .saveAsGif(
                    fileName.endsWith(".gif") ? fileName : fileName + ".gif");
          }
        }
      }
    });
  }

  private void createZoomCombo() {
    zoomCombo =
        new JComboBox(new Object[]{"300%", "200%", "150%", "100%", "75%",
            "50%", "25%"});
    zoomCombo.setSelectedIndex(3);
    zoomCombo.setToolTipText("Zoom");
    zoomCombo.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        ContextViewRenderer.zoom(zoomCombo.getSelectedItem().toString());
      }
    });
  }

  public void refreshView() {
    ContextViewRenderer.paintContext(this.projectPath, this.graphPanel,
        new ContextViewCallbackImpl(this, this.projectPath));
  }

  public JPanel getRootComponent() {
    return rootComponent;
  }

  public Project getProject() {
    return project;
  }
}
