/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.util.io.FileUtil;
import com.intellij.openapi.vfs.VirtualFile;
import midstream.core.handler.BeanHandler;
import midstream.util.IdeUtils;

import javax.swing.JOptionPane;
import java.io.File;
import java.io.IOException;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class CreateBeanDefinitionAction extends AnAction {

  public void update(AnActionEvent event) {
    VirtualFile selectedFile =
        (VirtualFile) event.getDataContext()
            .getData(DataConstants.VIRTUAL_FILE);
    Project project =
        (Project) event.getDataContext().getData(DataConstants.PROJECT);

    event.getPresentation().setVisible(IdeUtils.isPluginProject(project) &&
        selectedFile != null && "java".equals(selectedFile.getExtension()) &&
        !FileUtil.toSystemIndependentName(selectedFile.getPath())
            .contains("test/src"));
  }

  public void actionPerformed(AnActionEvent e) {
    final Project project =
        (Project) e.getDataContext().getData(DataConstants.PROJECT);

    if (project == null) {
      JOptionPane.showMessageDialog(null, "The current project is invalid.",
          "Error Creating Bean Definition", JOptionPane.ERROR_MESSAGE);
      return;
    }

    final String projectPath = project.getProjectFilePath().substring(0,
        project.getProjectFilePath().indexOf(
            project.getProjectFile().getName()));
    VirtualFile sourceFile =
        (VirtualFile) e.getDataContext().getData(DataConstants.VIRTUAL_FILE);

    if (sourceFile == null) {
      JOptionPane.showMessageDialog(null,
          "Unable to create a bean definition for the selected file.",
          "Error Creating Bean Definition", JOptionPane.ERROR_MESSAGE);
      return;
    }

    final String classFile =
        FileUtil.toSystemIndependentName(sourceFile.getPath())
            .replaceFirst(FileUtil.toSystemIndependentName(
                projectPath + "src" + File.separatorChar), "")
            .replace('/', '.').replace(".java", "");

    Runnable runner = new Runnable() {
      public void run() {
        CreateBeanDefinitionDialog dialog = new CreateBeanDefinitionDialog(
            IdeUtils.suggestParentWindow(project), project,
            projectPath, classFile);

        dialog.setVisible(true);

        if (dialog.isOk()) {
          try {
            new BeanHandler(projectPath)
                .createBean((String) dialog.getPageValues().get(
                    "bean.name"), classFile, (Map<String, String>) dialog
                    .getPageValues().get("properties"));
          } catch (IOException ex) {
            JOptionPane.showMessageDialog(null, ex,
                "Error Creating Bean Definition", JOptionPane.ERROR_MESSAGE);
            Logger.getInstance(this.getClass().getName()).error(ex);
          }

          JOptionPane.showMessageDialog(null,
              "A new bean named '" + dialog.getPageValues().get("bean.name")
                  + "' has been added to the application's context.",
              "Create Bean Definition",
              JOptionPane.INFORMATION_MESSAGE);
          IdeUtils.openFile(project, projectPath + File.separatorChar + "conf"
              + File.separatorChar + "applicationContext.xml");
        }
      }
    };

    IdeUtils.rebuildProject(project, runner);
  }
}
