/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.vfs.VirtualFile;
import midstream.core.handler.MdsTestHandler;
import midstream.util.IdeUtils;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import java.io.IOException;

/**
 * @author Juan Hall
 */
public class CreateMdsBeanShellScriptAction extends AnAction {

  public void update(AnActionEvent event) {
    VirtualFile selectedFile =
        (VirtualFile) event.getDataContext()
            .getData(DataConstants.VIRTUAL_FILE);
    Project project =
        (Project) event.getDataContext().getData(DataConstants.PROJECT);

    event.getPresentation().setVisible(IdeUtils.isPluginProject(project) &&
        selectedFile != null && "xml".equals(selectedFile.getExtension()));
  }

  public void actionPerformed(AnActionEvent e) {
    VirtualFile mdsTestXml =
        (VirtualFile) e.getDataContext().getData(DataConstants.VIRTUAL_FILE);

    JFileChooser dd = new JFileChooser();
    dd.setDialogTitle("Save As MDS Test Scripts");
    dd.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

    if (dd.showSaveDialog(null) == JFileChooser
        .APPROVE_OPTION && dd.getSelectedFile() != null && mdsTestXml != null) {
      try {
        new MdsTestHandler(null).convertTestToScript(mdsTestXml.getPath(),
            dd.getSelectedFile().getAbsolutePath());
        JOptionPane.showMessageDialog(null,
            "'" + dd.getSelectedFile() + "' has been saved.",
            "Create MDS BeanShell Scripts",
            JOptionPane.INFORMATION_MESSAGE);
      } catch (IOException ex) {
        JOptionPane.showMessageDialog(null, ex,
            "Error Creating MDS Test Scripts", JOptionPane.ERROR_MESSAGE);
        Logger.getInstance(this.getClass().getName()).error(ex);
      }
    }
  }
}
