/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.diagnostic.Logger;
import midstream.core.MdsTest;
import midstream.core.handler.MdsTestHandler;
import midstream.core.util.WizardUtils;
import midstream.util.IdeUtils;

import javax.swing.DefaultCellEditor;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Juan Hall
 */
public class MdsTestDialog extends JDialog {
  private JPanel contentPane;
  private JButton buttonOK;
  private JButton buttonCancel;
  private JComboBox serviceCombo;
  private JComboBox methodCombo;
  private JTable parameterTable;
  private JTabbedPane tabPane;
  private JTextField freeFormTestNameText;
  private JTextArea freeFormScriptText;
  private JTextField testNameText;
  private boolean ok;
  private String projectPath;
  private MdsTest selectedTest;
  private JButton freeFormTemplateButton;
  private JComboBox freeFormTemplateCombo;
  private JTextField postUrlText;
  private JButton postTemplateButton;
  private JComboBox postTemplateCombo;
  private JTextArea postTextArea;
  private JTextField postTestNameText;

  public MdsTestDialog(JFrame parent, String projectPath) {
    this(parent, projectPath, null);
  }

  public MdsTestDialog(JFrame parent, String projectPath, String test) {
    super(parent, "Create New MDS Test", true);

    try {
      if (test != null) {
        this.selectedTest = new MdsTestHandler(projectPath).parseTest(test);
      }
    } catch (IOException ex) {
      JOptionPane.showMessageDialog(null, ex, "Error Modifying Test",
          JOptionPane.ERROR_MESSAGE);
    }

    if (this.selectedTest != null) {
      this.setTitle("Modify MDS Test");
    }

    this.projectPath = projectPath;
    setContentPane(contentPane);
    getRootPane().setDefaultButton(buttonOK);

    buttonOK.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onOK();
      }
    });

    buttonCancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    });

// call onCancel() when cross is clicked
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        onCancel();
      }
    });

// call onCancel() on ESCAPE
    contentPane.registerKeyboardAction(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
        JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

    populateServiceCombo();
    serviceCombo.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent itemEvent) {
        populateMethodCombo();
      }
    });

    methodCombo.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent itemEvent) {
        populateParameterTable();
      }
    });

    parameterTable.setModel(new DefaultTableModel() {
      public boolean isCellEditable(int row, int col) {
        return col == 1;
      }
    });

    ((DefaultTableModel) parameterTable.getModel()).addColumn("Parameter");
    ((DefaultTableModel) parameterTable.getModel()).addColumn("Value");

    this.tabPane.setTitleAt(0, "Java/HTTP");
    this.tabPane.setTitleAt(1, "HTTP Post");
    this.tabPane.setTitleAt(2, "Free Form");

    if (selectedTest != null &&
        selectedTest.getTestType().equals(MdsTest.TestType.REMOTE)) {
      this.tabPane.setEnabledAt(1, false);
      this.tabPane.setEnabledAt(2, false);

      this.testNameText.setText(this.selectedTest.getTestName());
      this.testNameText.setEnabled(false);

      this.serviceCombo.setSelectedItem(this.selectedTest.getServiceName());
      this.serviceCombo.setEnabled(false);

      this.methodCombo.setModel(new DefaultComboBoxModel(
          new String[]{this.selectedTest.getMethodName()}));
      this.methodCombo.setEnabled(false);

      Object[][] dataVector = new Object[selectedTest.getParameters().size()][];

      for (int i = 0; i < selectedTest.getParameters().size(); ++i) {
        dataVector[i] = selectedTest.getParameters().get(i);
      }

      initializeParameterTable(dataVector);
    }

    if (selectedTest != null &&
        selectedTest.getTestType().equals(MdsTest.TestType.HTTP_POST)) {
      this.tabPane.setEnabledAt(0, false);
      this.tabPane.setEnabledAt(2, false);
      this.tabPane.setSelectedIndex(1);

      this.postTestNameText.setText(this.selectedTest.getTestName());
      this.postTestNameText.setEnabled(false);

      this.postUrlText.setText(this.selectedTest.getPostUrl());
      this.postTextArea.setText(this.selectedTest.getPostPayload());
    }

    if (selectedTest != null &&
        selectedTest.getTestType().equals(MdsTest.TestType.FREE_FORM)) {
      this.tabPane.setEnabledAt(0, false);
      this.tabPane.setEnabledAt(1, false);
      this.tabPane.setSelectedIndex(2);

      freeFormTestNameText.setText(selectedTest.getTestName());
      freeFormTestNameText.setEnabled(false);

      freeFormScriptText.setText(selectedTest.getScript());
    }

    postTemplateCombo.setModel(
        new DefaultComboBoxModel(
            new Object[]{"<Select Template>", "XML-RPC",
                "Custom XML/HTTP", "SOAP"}));

    freeFormTemplateCombo.setModel(
        new DefaultComboBoxModel(
            new Object[]{"<Select Template>", "Queue Sender",
                "Topic Publisher", "Socket Client"}));

    final JDialog thisDialog = this;
    postTemplateButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        String item = postTemplateCombo.getSelectedItem() == null ? null :
            postTemplateCombo.getSelectedItem().toString();
        String file = null;

        if ("XML-RPC".equals(item)) {
          file = "xml_rpc_packet.template";
        } else if ("Custom XML/HTTP".equals(item)) {
          file = "xml_http_packet.template";
        } else if ("SOAP".equals(item)) {
          file = "soap_packet.template";
        }

        if (file != null) {
          try {
            postTextArea
                .setText(WizardUtils.readStream(this.getClass().getClassLoader()
                    .getResourceAsStream(
                    "midstream/core/templates/components/"
                        + file)));
          } catch (IOException ex) {
            JOptionPane.showMessageDialog(thisDialog,
                "Unable to open template file:" + ex.getMessage(),
                "Error", JOptionPane.ERROR_MESSAGE);
          }
        }
      }
    });

    freeFormTemplateButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        String item = freeFormTemplateCombo.getSelectedItem() == null ? null :
            freeFormTemplateCombo.getSelectedItem().toString();
        String file = null;

        if ("Queue Sender".equals(item)) {
          file = "queue_producer.template";
        } else if ("Topic Publisher".equals(item)) {
          file = "topic_producer.template";
        } else if ("Socket Client".equals(item)) {
          file = "socket_client.template";
        }

        if (file != null) {
          try {
            freeFormScriptText
                .setText(WizardUtils.readStream(this.getClass().getClassLoader()
                    .getResourceAsStream(
                    "midstream/core/templates/components/"
                        + file)));
          } catch (IOException ex) {
            JOptionPane.showMessageDialog(thisDialog,
                "Unable to open template file:" + ex.getMessage(),
                "Error", JOptionPane.ERROR_MESSAGE);
          }
        }
      }
    });

    IdeUtils.packAndCenter(this);
  }

  private void populateServiceCombo() {
    try {
      List<String> services = new MdsTestHandler(projectPath).getServices();

      services.add(0, "<Select Service>");

      serviceCombo.setModel(
          new DefaultComboBoxModel(services.toArray(new String[0])));
    } catch (Exception ex) {
      Logger.getInstance(this.getClass().getName()).error(ex);
    }
  }

  private void populateMethodCombo() {
    List<String> names = new ArrayList<String>(5);

    names.add("<Select Method>");

    if (serviceCombo.getSelectedIndex() != 0) {
      names.addAll(new MdsTestHandler(projectPath).getMethodNames(
          (String) serviceCombo.getSelectedItem()));
    }

    methodCombo
        .setModel(new DefaultComboBoxModel(names.toArray(new String[0])));
  }

  private void populateParameterTable() {
    parameterTable.removeAll();
    ((DefaultTableModel) parameterTable.getModel())
        .setDataVector(new Object[0][], new Object[]{"Parameter", "Value"});

    if (methodCombo.getSelectedIndex() == 0) {
      return;
    }

    List<String> paramTypes = new MdsTestHandler(projectPath).getParameterTypes(
        (String) serviceCombo.getSelectedItem(),
        methodCombo.getSelectedIndex() - 1);

    if (paramTypes != null && !paramTypes.isEmpty()) {
      parameterTable.setEnabled(true);

      Object[][] dataVector = new Object[paramTypes.size()][];

      for (int i = 0; i < paramTypes.size(); ++i) {
        dataVector[i] = new Object[]{paramTypes.get(i), ""};
      }

      initializeParameterTable(dataVector);
    } else {
      parameterTable.setEnabled(false);
    }
  }

  private void initializeParameterTable(Object[][] dataVector) {
    ((DefaultTableModel) parameterTable.getModel())
        .setDataVector(dataVector, new Object[]{"Parameter", "Value"});
    final JButton tableButton = new JButton("...");
    final JDialog thisDialog = this;
    tableButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        int row = parameterTable.getSelectedRow();
        int column = parameterTable.getSelectedColumn();
        MdsTestParameterEditDialog dialog =
            new MdsTestParameterEditDialog(thisDialog);
        dialog
            .setParameterType(parameterTable.getValueAt(row, 0).toString());
        dialog.setParameterValue(
            parameterTable.getValueAt(row, column).toString());
        dialog.setVisible(true);

        if (dialog.isOk()) {
          parameterTable
              .setValueAt(dialog.getParameterValue(), row, column);
        }
      }
    });
    parameterTable.getColumnModel().getColumn(1)
        .setCellEditor(new DefaultCellEditor(new JCheckBox()) {
          public Component getTableCellEditorComponent(JTable jTable,
                                                       Object object,
                                                       boolean b, int i,
                                                       int i1) {
            return tableButton;
          }
        });
    parameterTable.getColumnModel().getColumn(1)
        .setCellRenderer(new TableCellRenderer() {
          public Component getTableCellRendererComponent(JTable jTable,
                                                         Object object,
                                                         boolean b,
                                                         boolean b1, int i,
                                                         int i1) {
            return tableButton;
          }
        });
  }

  private void onOK() {
    if (this.tabPane.getSelectedIndex() == 0
        || (this.selectedTest != null &&
        MdsTest.TestType.REMOTE.equals(this.selectedTest.getTestType()))) {
      if ("".equals(this.testNameText.getText().trim())) {
        JOptionPane.showMessageDialog(this, "A test name must be entered.",
            "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }
      if (this.selectedTest == null
          && new File(this.projectPath + File.separatorChar + "mds" + File
          .separatorChar + this.testNameText.getText().trim() + ".xml")
          .exists()) {
        JOptionPane.showMessageDialog(this, "A test named '" +
            this.testNameText.getText().trim() + "' already exists.", "Error",
            JOptionPane.ERROR_MESSAGE);
        return;
      }
      if (this.serviceCombo.getSelectedIndex() < 1) {
        JOptionPane.showMessageDialog(this, "A service must be selected.",
            "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }
      if (this.selectedTest == null && this.methodCombo.getSelectedIndex() < 1)
      {
        JOptionPane
            .showMessageDialog(this, "A method must be selected.", "Error",
                JOptionPane.ERROR_MESSAGE);
        return;
      }
    } else if (this.tabPane.getSelectedIndex() == 1
        || (this.selectedTest != null &&
        MdsTest.TestType.HTTP_POST.equals(this.selectedTest.getTestType()))) {
      if ("".equals(this.postTestNameText.getText().trim())) {
        JOptionPane.showMessageDialog(this, "A test name must be entered.",
            "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }
      if ("".equals(this.postUrlText.getText().trim())) {
        JOptionPane.showMessageDialog(this, "A URL must be entered.",
            "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }
      if (this.selectedTest == null
          && new File(this.projectPath + File.separatorChar + "mds" + File
          .separatorChar + this.postTestNameText.getText().trim() + ".xml")
          .exists()) {
        JOptionPane.showMessageDialog(this, "A test named '" +
            this.postTestNameText.getText().trim() + "' already exists.",
            "Error",
            JOptionPane.ERROR_MESSAGE);
        return;
      }
      if ("".equals(this.postTextArea.getText().trim())) {
        JOptionPane.showMessageDialog(this, "Payload text must be entered.",
            "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }
    } else if (this.tabPane.getSelectedIndex() == 2
        || (this.selectedTest != null &&
        MdsTest.TestType.FREE_FORM.equals(this.selectedTest.getTestType()))) {
      if ("".equals(this.freeFormTestNameText.getText().trim())) {
        JOptionPane.showMessageDialog(this, "A test name must be entered.",
            "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }
      if (this.selectedTest == null
          && new File(this.projectPath + File.separatorChar + "mds" + File
          .separatorChar + this.freeFormTestNameText.getText().trim() + ".xml")
          .exists()) {
        JOptionPane.showMessageDialog(this, "A test named '" +
            this.freeFormTestNameText.getText().trim() + "' already exists.",
            "Error",
            JOptionPane.ERROR_MESSAGE);
        return;
      }
      if ("".equals(this.freeFormScriptText.getText().trim())) {
        JOptionPane.showMessageDialog(this, "Script text must be entered.",
            "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }
    }

    ok = true;
    dispose();
  }

  private void onCancel() {
    ok = false;
    dispose();
  }

  public boolean isOk() {
    return ok;
  }

  public String getTestName() {
    return this.testNameText.getText().trim();
  }

  public String getFreeFormTestName() {
    return this.freeFormTestNameText.getText().trim();
  }

  public String getFreeFormScript() {
    return this.freeFormScriptText.getText().trim();
  }

  public String getService() {
    return this.serviceCombo.getSelectedItem().toString();
  }

  public String getMethod() {
    return this.methodCombo.getSelectedItem().toString();
  }

  public String[][] getParameterValues() {
    String[][] values = new String[this.parameterTable.getRowCount()][2];
    for (int i = 0; i < this.parameterTable.getRowCount(); ++i) {
      values[i][0] = this.parameterTable.getValueAt(i, 0).toString();
      values[i][1] = this.parameterTable.getValueAt(i, 1).toString();
    }
    return values;
  }

  public String getPostTestName() {
    return this.postTestNameText.getText().trim();
  }

  public String getPostUrl() {
    return this.postUrlText.getText().trim();
  }

  public String getPostPayload() {
    return this.postTextArea.getText().trim();
  }

  public MdsTest.TestType getTestType() {
    return this.tabPane.getSelectedIndex() == 0 ? MdsTest.TestType.REMOTE :
        this
            .tabPane.getSelectedIndex() == 1 ? MdsTest.TestType.HTTP_POST :
            MdsTest.TestType.FREE_FORM;
  }
}