/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.components.ProjectComponent;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.wm.ToolWindowAnchor;
import com.intellij.openapi.wm.ToolWindowManager;
import midstream.util.IdeUtils;

import javax.swing.ImageIcon;

/**
 * @author Juan Hall
 */
public class MdsTestRunnerProjectComponent implements ProjectComponent {
  private static final String WINDOW_NAME = "MDS Test Runner";
  private Project project;
  private MdsTestRunnerForm form;

  public MdsTestRunnerProjectComponent(Project project) {
    this.project = project;
  }

  public void initComponent() {
  }

  public void disposeComponent() {
  }

  public String getComponentName() {
    return "MdsTestRunnerProjectComponent";
  }

  public void projectOpened() {
    if (form == null && IdeUtils.isPluginProject(project)) {
      form = new MdsTestRunnerForm(project);
      ToolWindowManager.getInstance(project).registerToolWindow(
          WINDOW_NAME, form.getRootComponent(), ToolWindowAnchor.RIGHT);
      ToolWindowManager.getInstance(project).getToolWindow(WINDOW_NAME)
          .setIcon(new ImageIcon(
              getClass().getClassLoader().getResource(
                  "midstream/core/resources/icons/action.gif")));
    }
  }

  public void projectClosed() {
    if (ToolWindowManager.getInstance(project).getToolWindow(WINDOW_NAME) !=
        null && IdeUtils.isPluginProject(project)) {
      ToolWindowManager.getInstance(project)
          .unregisterToolWindow(WINDOW_NAME);
    }

    form = null;
  }
}
