/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.project.Project;
import midstream.core.handler.CacheManagerBeanHandler;
import midstream.core.util.WizardUtils;
import midstream.util.IdeUtils;

import javax.swing.JOptionPane;
import java.io.File;
import java.io.IOException;

/**
 * @author Juan Hall
 */
public class NewCacheManagerBeanAction extends AnAction {

  public void update(AnActionEvent event) {
    Project project =
        (Project) event.getDataContext().getData(DataConstants.PROJECT);

    event.getPresentation().setVisible(IdeUtils.isPluginProject(project));
  }

  public void actionPerformed(AnActionEvent e) {
    Project project =
        (Project) e.getDataContext().getData(DataConstants.PROJECT);

    if (project == null) {
      JOptionPane.showMessageDialog(null, "The current project is invalid.",
          "Error Creating Cache Manager Bean", JOptionPane.ERROR_MESSAGE);
      return;
    }

    try {
      String projectPath = project.getProjectFilePath().substring(0,
          project.getProjectFilePath().indexOf(
              project.getProjectFile().getName()));
      CacheManagerBeanHandler cacheManagerBeanHandler =
          new CacheManagerBeanHandler(projectPath);
      cacheManagerBeanHandler.createServiceContextEntry();
      cacheManagerBeanHandler.createCacheConfigFiles();
      cacheManagerBeanHandler.createCacheFactoryBeanClassTest();
      cacheManagerBeanHandler.updateBuildFile();

      IdeUtils.createLibraries(project, projectPath,
          CacheManagerBeanHandler.LIBRARIES);
      WizardUtils
          .updateBuildFileWarLibraries(projectPath,
              CacheManagerBeanHandler.LIBRARIES,
              true);

      JOptionPane.showMessageDialog(null,
          "An cache manager bean has been added to the 'applicationContext.xml' file.",
          "Create Cache Manager Bean", JOptionPane.INFORMATION_MESSAGE);

      IdeUtils.reloadProject();
      IdeUtils
          .openFile(project, projectPath + File.separatorChar + "conf" + File
              .separatorChar + "applicationContext.xml");
    } catch (IOException ex) {
      JOptionPane.showMessageDialog(null, ex,
          "Error Creating Cache Manager Bean", JOptionPane.ERROR_MESSAGE);
      Logger.getInstance(this.getClass().getName()).error(ex);
    }
  }
}
