/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.project.Project;
import midstream.core.handler.DaoHandler;
import midstream.util.IdeUtils;

import javax.swing.JOptionPane;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class NewDaoAction extends AnAction {
  private NewDaoDialog dialog;
  private String javaPackage;
  private String daoName;
  private String sql;
  private String dataSource;

  public void update(AnActionEvent event) {
    Project project =
        (Project) event.getDataContext().getData(DataConstants.PROJECT);

    event.getPresentation().setVisible(IdeUtils.isPluginProject(project));
  }

  public void actionPerformed(AnActionEvent e) {
    Project project =
        (Project) e.getDataContext().getData(DataConstants.PROJECT);

    if (project == null) {
      JOptionPane.showMessageDialog(null, "The current project is invalid.",
          "Error Creating Data Access Object", JOptionPane.ERROR_MESSAGE);
      return;
    }

    if (getDialog() == null) {
      setDialog(new NewDaoDialog(IdeUtils.suggestParentWindow(project)));
    }

    getDialog().setData(this);
    getDialog().setVisible(true);

    if (getDialog().isOk()) {
      getDialog().getData(this);

      try {
        final String projectPath = project.getProjectFilePath().substring(0,
            project.getProjectFilePath().indexOf(
                project.getProjectFile().getName()));

        Map<String, String> pageValues = new HashMap<String, String>(5);
        pageValues.put("java.package", this.getJavaPackage().trim());
        pageValues.put("dao.name", this.getDaoName().trim());
        pageValues.put("datasource", this.getDataSource().trim());
        pageValues.put("sql", this.getSql().trim());

        DaoHandler daoHandler = new DaoHandler(projectPath);
        daoHandler.createDaoInterface(pageValues);
        String daoClass = daoHandler.createDaoClass(pageValues);
        daoHandler.createDaoClassTest(pageValues);
        daoHandler.createServiceContextEntry(pageValues);
        daoHandler.updateDeploymentDescriptor(pageValues);

        IdeUtils.createLibraries(project, projectPath, DaoHandler.LIBRARIES);

        IdeUtils.reloadProject();
        IdeUtils.openFile(project, projectPath + File.separatorChar + daoClass);
      } catch (IOException ex) {
        JOptionPane.showMessageDialog(null, ex,
            "Error Creating Data Access Object", JOptionPane.ERROR_MESSAGE);
        Logger.getInstance(this.getClass().getName()).error(ex);
      }
    }
  }

  public String getJavaPackage() {
    return javaPackage;
  }

  public void setJavaPackage(final String javaPackage) {
    this.javaPackage = javaPackage;
  }

  public String getDaoName() {
    return daoName;
  }

  public void setDaoName(final String daoName) {
    this.daoName = daoName;
  }

  public String getSql() {
    return sql;
  }

  public void setSql(final String sql) {
    this.sql = sql;
  }

  public String getDataSource() {
    return dataSource;
  }

  public void setDataSource(String dataSource) {
    this.dataSource = dataSource;
  }

  public NewDaoDialog getDialog() {
    return dialog;
  }

  public void setDialog(NewDaoDialog dialog) {
    this.dialog = dialog;
  }
}
