/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.project.Project;
import midstream.core.handler.HttpClientHandler;
import midstream.core.util.WizardUtils;
import midstream.util.IdeUtils;

import javax.swing.JOptionPane;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class NewHttpClientAction extends AnAction {
  private NewHttpClientDialog dialog;
  private String javaPackage;
  private String clientName;
  private String connectionTimeout = "10";
  private String readTimeout = "10";

  public void update(AnActionEvent event) {
    Project project =
        (Project) event.getDataContext().getData(DataConstants.PROJECT);

    event.getPresentation().setVisible(IdeUtils.isPluginProject(project));
  }

  public void actionPerformed(AnActionEvent e) {
    Project project =
        (Project) e.getDataContext().getData(DataConstants.PROJECT);

    if (project == null) {
      JOptionPane.showMessageDialog(null, "The current project is invalid.",
          "Error Creating HTTP Client", JOptionPane.ERROR_MESSAGE);
      return;
    }

    if (this.dialog == null) {
      this.dialog =
          new NewHttpClientDialog(IdeUtils.suggestParentWindow(project));
    }

    dialog.setData(this);
    dialog.setVisible(true);

    if (dialog.isOk()) {
      dialog.getData(this);

      try {
        final String projectPath = project.getProjectFilePath().substring(0,
            project.getProjectFilePath().indexOf(
                project.getProjectFile().getName()));

        Map<String, String> pageValues = new HashMap<String, String>(7);
        pageValues.put("java.package", getJavaPackage());
        pageValues.put("client.name", getClientName());
        pageValues
            .put("connection.timeout", getConnectionTimeout());
        pageValues.put("read.timeout", getReadTimeout());

        HttpClientHandler httpClientHandler =
            new HttpClientHandler(projectPath);
        String clientClass =
            httpClientHandler.createClientClass(pageValues);
        httpClientHandler.createClientClassTest(pageValues);
        httpClientHandler.createContextEntry(pageValues);
        IdeUtils
            .createLibraries(project, projectPath, HttpClientHandler.LIBRARIES);
        WizardUtils.updateBuildFileWarLibraries(projectPath,
            HttpClientHandler.LIBRARIES, true);

        IdeUtils.reloadProject();
        IdeUtils
            .openFile(project, projectPath + File.separatorChar + clientClass);
      } catch (IOException ex) {
        JOptionPane.showMessageDialog(null, ex,
            "Error Creating HTTP Client", JOptionPane.ERROR_MESSAGE);
        Logger.getInstance(this.getClass().getName()).error(ex);
      }
    }
  }

  public NewHttpClientDialog getDialog() {
    return dialog;
  }

  public void setDialog(NewHttpClientDialog dialog) {
    this.dialog = dialog;
  }

  public String getJavaPackage() {
    return javaPackage;
  }

  public void setJavaPackage(String javaPackage) {
    this.javaPackage = javaPackage;
  }

  public String getClientName() {
    return clientName;
  }

  public void setClientName(String clientName) {
    this.clientName = clientName;
  }

  public String getConnectionTimeout() {
    return connectionTimeout;
  }

  public void setConnectionTimeout(String connectionTimeout) {
    this.connectionTimeout = connectionTimeout;
  }

  public String getReadTimeout() {
    return readTimeout;
  }

  public void setReadTimeout(String readTimeout) {
    this.readTimeout = readTimeout;
  }
}
