/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.project.Project;
import midstream.core.handler.JmsActorHandler;
import midstream.util.IdeUtils;

import javax.swing.JOptionPane;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class NewJmsMessageProducerAction extends AnAction {
  private NewJmsMessageProducerDialog dialog;
  private String destinationName;
  private String userName;
  private String password;
  private String javaPackage;
  private String actorName;
  private boolean queue = true;
  private String connectionFactoryClass = "";
  private String brokerUrl = "";
  private String channel = "";
  private String hostName = "";
  private String port = "";
  private String queueManager = "";
  private String transportType = "";
  private String connectionFactoryJndiName = "";
  private String namingFactoryInitial = "";
  private String providerUrl = "";
  private String securityPrincipal = "";
  private String securityCredentials = "";

  public void update(AnActionEvent event) {
    Project project =
        (Project) event.getDataContext().getData(DataConstants.PROJECT);

    event.getPresentation().setVisible(IdeUtils.isPluginProject(project));
  }

  public void actionPerformed(AnActionEvent e) {
    Project project =
        (Project) e.getDataContext().getData(DataConstants.PROJECT);

    if (project == null) {
      JOptionPane.showMessageDialog(null, "The current project is invalid.",
          "Error Creating JMS Message Producer", JOptionPane.ERROR_MESSAGE);
      return;
    }

    if (getDialog() == null) {
      setDialog(
          new NewJmsMessageProducerDialog(
              IdeUtils.suggestParentWindow(project)));
    }

    getDialog().setData(this);
    getDialog().setVisible(true);

    if (getDialog().isOk()) {
      getDialog().getData(this);

      try {
        final String projectPath = project.getProjectFilePath().substring(0,
            project.getProjectFilePath().indexOf(
                project.getProjectFile().getName()));

        Map<String, String> pageValues = new HashMap<String, String>(5);
        pageValues.put("java.package", this.getJavaPackage().trim());
        pageValues.put("actor.name", this.getActorName().trim());
        pageValues.put("actor.type", "Producer");
        pageValues.put("destination.name", this.getDestinationName().trim());
        pageValues.put("user.name", this.getUserName().trim());
        pageValues.put("password", this.getPassword().trim());
        pageValues.put("connection.factory.class",
            this.getConnectionFactoryClass().trim());
        pageValues.put("broker.url", this.getBrokerUrl().trim());
        pageValues.put("channel", this.getChannel().trim());
        pageValues.put("host.name", this.getHostName().trim());
        pageValues.put("port", this.getPort().trim());
        pageValues.put("queue.manager", this.getQueueManager().trim());
        pageValues.put("transport.type", this.getTransportType().trim());
        pageValues.put("connection.factory.jndi.name",
            this.getConnectionFactoryJndiName().trim());
        pageValues.put("naming.factory.initial",
            this.getNamingFactoryInitial().trim());
        pageValues.put("provider.url", this.getProviderUrl().trim());
        pageValues
            .put("security.principal", this.getSecurityPrincipal().trim());
        pageValues
            .put("security.credentials", this.getSecurityCredentials().trim());
        pageValues.put("is.queue", Boolean.toString(this.isQueue()));

        JmsActorHandler jmsActorHandler = new JmsActorHandler(projectPath);
        String serviceClass =
            jmsActorHandler.createProducerClass(pageValues);
        jmsActorHandler.createProducerClassTest(pageValues);
        jmsActorHandler.createProducerConfiguration(pageValues);
        IdeUtils.createLibraries(project, projectPath,
            JmsActorHandler.LIBRARIES);

        IdeUtils.reloadProject();
        IdeUtils
            .openFile(project, projectPath + File.separatorChar + serviceClass);

        JOptionPane.showMessageDialog(null,
            "Remember to add the necessary message broker libraries to the project.\n"
                +
                "'Right click on the project -> Project Dependencies...' to search available libraries.",
            "Dependency Reminder",
            JOptionPane.INFORMATION_MESSAGE);

      } catch (IOException ex) {
        JOptionPane
            .showMessageDialog(null, ex, "Error Creating JMS Message Producer",
                JOptionPane.ERROR_MESSAGE);
        Logger.getInstance(this.getClass().getName()).error(ex);
      }
    }
  }

  public NewJmsMessageProducerDialog getDialog() {
    return dialog;
  }

  public void setDialog(NewJmsMessageProducerDialog dialog) {
    this.dialog = dialog;
  }

  public String getDestinationName() {
    return destinationName;
  }

  public void setDestinationName(String destinationName) {
    this.destinationName = destinationName;
  }

  public String getUserName() {
    return userName;
  }

  public void setUserName(String userName) {
    this.userName = userName;
  }

  public String getPassword() {
    return password;
  }

  public void setPassword(String password) {
    this.password = password;
  }

  public String getJavaPackage() {
    return javaPackage;
  }

  public void setJavaPackage(String javaPackage) {
    this.javaPackage = javaPackage;
  }

  public String getActorName() {
    return actorName;
  }

  public void setActorName(String actorName) {
    this.actorName = actorName;
  }

  public boolean isQueue() {
    return queue;
  }

  public void setQueue(boolean queue) {
    this.queue = queue;
  }

  public String getConnectionFactoryClass() {
    return connectionFactoryClass;
  }

  public void setConnectionFactoryClass(String connectionFactoryClass) {
    this.connectionFactoryClass = connectionFactoryClass;
  }

  public String getBrokerUrl() {
    return brokerUrl;
  }

  public void setBrokerUrl(String brokerUrl) {
    this.brokerUrl = brokerUrl;
  }

  public String getChannel() {
    return channel;
  }

  public void setChannel(String channel) {
    this.channel = channel;
  }

  public String getHostName() {
    return hostName;
  }

  public void setHostName(String hostName) {
    this.hostName = hostName;
  }

  public String getPort() {
    return port;
  }

  public void setPort(String port) {
    this.port = port;
  }

  public String getQueueManager() {
    return queueManager;
  }

  public void setQueueManager(String queueManager) {
    this.queueManager = queueManager;
  }

  public String getTransportType() {
    return transportType;
  }

  public void setTransportType(String transportType) {
    this.transportType = transportType;
  }

  public String getConnectionFactoryJndiName() {
    return connectionFactoryJndiName;
  }

  public void setConnectionFactoryJndiName(String connectionFactoryJndiName) {
    this.connectionFactoryJndiName = connectionFactoryJndiName;
  }

  public String getNamingFactoryInitial() {
    return namingFactoryInitial;
  }

  public void setNamingFactoryInitial(String namingFactoryInitial) {
    this.namingFactoryInitial = namingFactoryInitial;
  }

  public String getProviderUrl() {
    return providerUrl;
  }

  public void setProviderUrl(String providerUrl) {
    this.providerUrl = providerUrl;
  }

  public String getSecurityPrincipal() {
    return securityPrincipal;
  }

  public void setSecurityPrincipal(String securityPrincipal) {
    this.securityPrincipal = securityPrincipal;
  }

  public String getSecurityCredentials() {
    return securityCredentials;
  }

  public void setSecurityCredentials(String securityCredentials) {
    this.securityCredentials = securityCredentials;
  }
}
