/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import midstream.util.IdeUtils;

import javax.swing.ButtonGroup;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.table.DefaultTableModel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * @author Juan Hall
 */
public class NewJmsReceiverDialog extends JDialog {
  private JPanel contentPane;
  private JButton buttonOK;
  private JButton buttonCancel;
  private JTextField javaPackageField;
  private JTextField receiverNameField;
  private JTable jndiInfoTable;
  private JTextField destinationNameField;
  private JRadioButton queueButton;
  private JRadioButton topicButton;
  private JTextField userNameField;
  private JTextField passwordField;
  private boolean ok = false;
  private JTextField listenerCountText;

  public NewJmsReceiverDialog(JFrame parent) {
    super(parent, "New JMS Receiver", true);
    setContentPane(contentPane);
    getRootPane().setDefaultButton(buttonOK);

    buttonOK.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onOK();
      }
    });

    buttonCancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    });

// call onCancel() when cross is clicked
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        onCancel();
      }
    });

// call onCancel() on ESCAPE
    contentPane.registerKeyboardAction(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
        JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

    ButtonGroup group = new ButtonGroup();
    group.add(queueButton);
    group.add(topicButton);

    DefaultCellEditor cellEditor = new DefaultCellEditor(new JTextField());
    cellEditor.setClickCountToStart(1);

    DefaultTableModel model = new DefaultTableModel() {
      public boolean isCellEditable(int row, int col) {
        return col > 0;
      }
    };
    model.addColumn("");
    model.addColumn("");
    model.addRow(new Object[]{"Connection Factory JNDI Name", ""});
    model.addRow(new Object[]{"Naming Factory Initial", ""});
    model.addRow(new Object[]{"Provider URL", ""});
    model.addRow(new Object[]{"Security Principal", ""});
    model.addRow(new Object[]{"Security Credentials", ""});
    jndiInfoTable.setModel(model);
    jndiInfoTable.getColumnModel().getColumn(1).setCellEditor(cellEditor);

    IdeUtils.packAndCenter(this);
  }

  private void onOK() {
    if ("".equals(this.javaPackageField.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A Java package must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    if ("".equals(this.receiverNameField.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A receiver name must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    if ("".equals(this.listenerCountText.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A listener count must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    if ("".equals(this.destinationNameField.getText().trim())) {
      JOptionPane.showMessageDialog(this,
          "A destination (queue/topic name) must be entered.", "Error",
          JOptionPane.ERROR_MESSAGE);
      return;
    }
    if (IdeUtils.isTableEmpty(jndiInfoTable, 1)) {
      JOptionPane.showMessageDialog(this,
          "JNDI connection information must be entered.", "Error",
          JOptionPane.ERROR_MESSAGE);
      return;
    }

    ok = true;
    dispose();
  }

  private void onCancel() {
    ok = false;
    dispose();
  }

  public void setData(NewJmsReceiverAction data) {
    destinationNameField.setText(data.getDestinationName());
    userNameField.setText(data.getUserName());
    passwordField.setText(data.getPassword());
    javaPackageField.setText(data.getJavaPackage());
    receiverNameField.setText(data.getActorName());
    listenerCountText.setText(data.getListenerCount());

    if (data.isQueue()) {
      queueButton.setSelected(true);
    } else {
      topicButton.setSelected(true);
    }

    int i = 0;
    jndiInfoTable.setValueAt(data.getConnectionFactoryJndiName(), i++, 1);
    jndiInfoTable.setValueAt(data.getNamingFactoryInitial(), i++, 1);
    jndiInfoTable.setValueAt(data.getProviderUrl(), i++, 1);
    jndiInfoTable.setValueAt(data.getSecurityPrincipal(), i++, 1);
    jndiInfoTable.setValueAt(data.getSecurityCredentials(), i, 1);
  }

  public void getData(NewJmsReceiverAction data) {
    data.setDestinationName(destinationNameField.getText());
    data.setUserName(userNameField.getText());
    data.setPassword(passwordField.getText());
    data.setJavaPackage(javaPackageField.getText());
    data.setActorName(receiverNameField.getText());
    data.setQueue(queueButton.isSelected());
    data.setListenerCount(listenerCountText.getText());

    int i = 0;
    data
        .setConnectionFactoryJndiName(
            (String) jndiInfoTable.getValueAt(i++, 1));
    data.setNamingFactoryInitial((String) jndiInfoTable.getValueAt(i++, 1));
    data.setProviderUrl((String) jndiInfoTable.getValueAt(i++, 1));
    data.setSecurityPrincipal((String) jndiInfoTable.getValueAt(i++, 1));
    data.setSecurityCredentials((String) jndiInfoTable.getValueAt(i, 1));
  }

  public boolean isOk() {
    return ok;
  }
}
