/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.actionSystem.ActionManager;
import com.intellij.openapi.actionSystem.Anchor;
import com.intellij.openapi.actionSystem.Constraints;
import com.intellij.openapi.actionSystem.DefaultActionGroup;
import com.intellij.openapi.actionSystem.Separator;
import com.intellij.openapi.components.ApplicationComponent;
import midstream.core.handler.ProjectDependencyHandler;
import midstream.util.IdeUtils;

/**
 * @author Juan Hall
 */
public class NewProjectApplicationComponent implements ApplicationComponent {
  private String projectName;
  private String deploymentPrefix;
  private String deploymentVersion = "1.0";
  private NewProjectDialog dialog;
  private String projectLocation;
  private String projectJdk;

  public void initComponent() {
    IdeUtils.setSystemProperties();
    new ProjectDependencyHandler(null).updateAvailableDependencyList();

    if (getDialog() == null) {
      setDialog(new NewProjectDialog());
      getDialog().setData(this);
    }

    ((DefaultActionGroup) ActionManager.getInstance().getAction("NewGroup"))
        .add(Separator.getInstance(),
            new Constraints(Anchor.AFTER, "NewScheduledService"));
  }

  public void disposeComponent() {
    new ProjectDependencyHandler(null).cleanUpAvailableDependencyList();

    if (getDialog().isVisible()) {
      getDialog().setVisible(false);
    }
    setDialog(null);
  }

  public String getComponentName() {
    return "NewProjectApplicationComponent";
  }

  public String getProjectName() {
    return projectName;
  }

  public void setProjectName(final String projectName) {
    this.projectName = projectName;
  }

  public String getProjectLocation() {
    return projectLocation;
  }

  public void setProjectLocation(final String projectLocation) {
    this.projectLocation = projectLocation;
  }

  public String getDeploymentPrefix() {
    return deploymentPrefix;
  }

  public void setDeploymentPrefix(final String deploymentPrefix) {
    this.deploymentPrefix = deploymentPrefix;
  }

  public String getDeploymentVersion() {
    return deploymentVersion;
  }

  public void setDeploymentVersion(final String deploymentVersion) {
    this.deploymentVersion = deploymentVersion;
  }

  public void launchDialog() {
    getDialog().setVisible(true);
    getDialog().getData(this);
  }

  public boolean isOk() {
    return getDialog().isOk();
  }

  public NewProjectDialog getDialog() {
    return dialog;
  }

  public void setDialog(NewProjectDialog dialog) {
    this.dialog = dialog;
  }

  public String getProjectJdk() {
    return projectJdk;
  }

  public void setProjectJdk(String projectJdk) {
    this.projectJdk = projectJdk;
  }
}
