/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import midstream.util.IdeUtils;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * @author Juan Hall
 */
public class NewProjectJdkDialog extends JDialog {
  private JPanel contentPane;
  private JButton buttonOK;
  private JButton buttonCancel;
  private JTextField jdkNameText;
  private JTextField jdkPathText;
  private JButton browseButton;
  private boolean ok;

  public NewProjectJdkDialog(JDialog parent) {
    super(parent, "New Project JDK", true);
    setContentPane(contentPane);
    getRootPane().setDefaultButton(buttonOK);

    buttonOK.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onOK();
      }
    });

    buttonCancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    });

// call onCancel() when cross is clicked
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        onCancel();
      }
    });

// call onCancel() on ESCAPE
    contentPane.registerKeyboardAction(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
        JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

    final NewProjectJdkDialog thisDialog = this;
    browseButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        fileChooser.showOpenDialog(thisDialog);
        if (fileChooser.getSelectedFile() != null) {
          thisDialog.jdkPathText
              .setText(fileChooser.getSelectedFile().getAbsolutePath());
        }
      }
    });

    IdeUtils.packAndCenter(this);
  }

  private void onOK() {
    if ("".equals(this.jdkNameText.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A JDK name must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    if ("".equals(this.jdkPathText.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A JDK path must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }

    ok = true;
    dispose();
  }

  private void onCancel() {
    ok = false;
    dispose();
  }

  public void setData(NewProjectDialog data) {
    jdkPathText.setText(data.getProjectJdkPath());
    jdkNameText.setText(data.getProjectJdkName());
  }

  public void getData(NewProjectDialog data) {
    data.setProjectJdkPath(jdkPathText.getText());
    data.setProjectJdkName(jdkNameText.getText());
  }

  public boolean isOk() {
    return ok;
  }
}
