/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import midstream.util.IdeUtils;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * @author Juan Hall
 */
public class NewRemoteServiceDialog extends JDialog {
  private JPanel contentPane;
  private JButton buttonOK;
  private JButton buttonCancel;
  private JTextField javaPackageField;
  private JTextField serviceNameField;
  private boolean ok = false;
  private JRadioButton httpButton;
  private JRadioButton xmlRpcButton;
  private JRadioButton soapButton;
  private JRadioButton xmlHttpButton;
  private JTextField serviceMethodNameText;
  private JTextField requestNameText;
  private JTextField responseNameText;
  private JPanel xmlHttpPanel;

  public NewRemoteServiceDialog(JFrame parent) {
    super(parent, "New Remote Service", true);
    setContentPane(contentPane);
    getRootPane().setDefaultButton(buttonOK);

    buttonOK.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onOK();
      }
    });

    buttonCancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    });

// call onCancel() when cross is clicked
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        onCancel();
      }
    });

// call onCancel() on ESCAPE
    contentPane.registerKeyboardAction(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
        JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

    ButtonGroup group = new ButtonGroup();
    group.add(httpButton);
    group.add(xmlHttpButton);
    group.add(xmlRpcButton);
    group.add(soapButton);

    final JDialog thisDialog = this;
    xmlHttpButton.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent itemEvent) {
        xmlHttpPanel
            .setVisible(itemEvent.getStateChange() == ItemEvent.SELECTED);

        if (itemEvent.getStateChange() == ItemEvent.SELECTED) {
          IdeUtils.packAndCenter(thisDialog);
        }
      }
    });

    xmlHttpPanel.setVisible(false);

    IdeUtils.packAndCenter(this);
  }

  private void onOK() {
    if ("".equals(this.javaPackageField.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A Java package must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    if ("".equals(this.serviceNameField.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A service name must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    if (xmlHttpButton.isSelected()) {
      if ("".equals(serviceMethodNameText.getText().trim())) {
        JOptionPane
            .showMessageDialog(this, "A service method name must be entered.",
                "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }
      if ("".equals(requestNameText.getText().trim())) {
        JOptionPane.showMessageDialog(this, "A request name must be entered.",
            "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }
      if ("".equals(responseNameText.getText().trim())) {
        JOptionPane.showMessageDialog(this, "A response name must be entered.",
            "Error", JOptionPane.ERROR_MESSAGE);
        return;
      }
    }

    ok = true;
    dispose();
  }

  private void onCancel() {
    ok = false;
    dispose();
  }

  public void setData(NewRemoteServiceAction data) {
    javaPackageField.setText(data.getJavaPackage());
    serviceNameField.setText(data.getServiceName());

    if ("XML-HTTP".equals(data.getRemotingType())) {
      xmlHttpButton.setSelected(true);
      serviceMethodNameText.setText(data.getServiceMethodName());
      requestNameText.setText(data.getRequestName());
      responseNameText.setText(data.getResponseName());
    } else if ("SOAP".equals(data.getRemotingType())) {
      soapButton.setSelected(true);
    } else if ("XML-RPC".equals(data.getRemotingType())) {
      xmlRpcButton.setSelected(true);
    } else {
      httpButton.setSelected(true);
    }
  }

  public void getData(NewRemoteServiceAction data) {
    data.setJavaPackage(javaPackageField.getText());
    data.setServiceName(serviceNameField.getText());
    data.setRemotingType(xmlHttpButton.isSelected() ? "XML-HTTP" :
        soapButton.isSelected() ? "SOAP" :
            xmlRpcButton.isSelected() ? "XML-RPC" : "HTTP");

    if (xmlHttpButton.isSelected()) {
      data.setServiceMethodName(serviceMethodNameText.getText());
      data.setRequestName(requestNameText.getText());
      data.setResponseName(responseNameText.getText());
    }
  }

  public boolean isOk() {
    return ok;
  }
}
