/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.project.Project;
import midstream.core.handler.ScheduledServiceHandler;
import midstream.core.util.WizardUtils;
import midstream.util.IdeUtils;

import javax.swing.JOptionPane;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class NewScheduledServiceAction extends AnAction {
  private NewScheduledServiceDialog dialog;
  private String delay;
  private String period;
  private String javaPackage;
  private String serviceName;

  public void update(AnActionEvent event) {
    Project project =
        (Project) event.getDataContext().getData(DataConstants.PROJECT);

    event.getPresentation().setVisible(IdeUtils.isPluginProject(project));
  }

  public void actionPerformed(AnActionEvent e) {
    Project project =
        (Project) e.getDataContext().getData(DataConstants.PROJECT);

    if (project == null) {
      JOptionPane.showMessageDialog(null, "The current project is invalid.",
          "Error Creating Scheduled Service", JOptionPane.ERROR_MESSAGE);
      return;
    }

    if (this.dialog == null) {
      this.dialog =
          new NewScheduledServiceDialog(IdeUtils.suggestParentWindow(project));
    }

    dialog.setData(this);
    dialog.setVisible(true);

    if (dialog.isOk()) {
      dialog.getData(this);

      try {
        final String projectPath = project.getProjectFilePath().substring(0,
            project.getProjectFilePath().indexOf(
                project.getProjectFile().getName()));

        Map<String, String> pageValues = new HashMap<String, String>(4);
        pageValues.put("java.package", getJavaPackage());
        pageValues.put("service.name", getServiceName());
        pageValues.put("delay", getDelay());
        pageValues.put("period", getPeriod());

        ScheduledServiceHandler scheduledServiceHandler =
            new ScheduledServiceHandler(projectPath);
        String serviceClass = scheduledServiceHandler
            .createScheduledServiceClass(pageValues);
        scheduledServiceHandler
            .createScheduledServiceClassTest(pageValues);
        scheduledServiceHandler
            .createServiceContextEntry(pageValues);
        IdeUtils.createLibraries(project, projectPath,
            ScheduledServiceHandler.LIBRARIES);
        WizardUtils.updateBuildFileWarLibraries(projectPath,
            ScheduledServiceHandler.LIBRARIES, true);

        IdeUtils.reloadProject();
        IdeUtils
            .openFile(project, projectPath + File.separatorChar + serviceClass);
      } catch (IOException ex) {
        JOptionPane.showMessageDialog(null, ex,
            "Error Creating Scheduled Service", JOptionPane.ERROR_MESSAGE);
        Logger.getInstance(this.getClass().getName()).error(ex);
      }
    }
  }

  public String getDelay() {
    return delay;
  }

  public void setDelay(final String delay) {
    this.delay = delay;
  }

  public String getPeriod() {
    return period;
  }

  public void setPeriod(final String period) {
    this.period = period;
  }

  public String getJavaPackage() {
    return javaPackage;
  }

  public void setJavaPackage(final String javaPackage) {
    this.javaPackage = javaPackage;
  }

  public String getServiceName() {
    return serviceName;
  }

  public void setServiceName(final String serviceName) {
    this.serviceName = serviceName;
  }
}
