/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import midstream.util.IdeUtils;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * @author Juan Hall
 */
public class NewScheduledServiceDialog extends JDialog {
  private JPanel contentPane;
  private JButton buttonOK;
  private JButton buttonCancel;
  private JTextField javaPackageText;
  private JTextField serviceNameText;
  private JTextField delayText;
  private JTextField periodText;
  private boolean ok;

  public NewScheduledServiceDialog(JFrame parent) {
    super(parent, "New Scheduled Service", true);
    setContentPane(contentPane);
    getRootPane().setDefaultButton(buttonOK);

    buttonOK.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onOK();
      }
    });

    buttonCancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    });

// call onCancel() when cross is clicked
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        onCancel();
      }
    });

// call onCancel() on ESCAPE
    contentPane.registerKeyboardAction(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
        JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

    IdeUtils.packAndCenter(this);
  }

  private void onOK() {
    if ("".equals(this.javaPackageText.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A Java package must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    if ("".equals(this.serviceNameText.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A service name must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    if ("".equals(this.delayText.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A delay must be entered.", "Error",
          JOptionPane.ERROR_MESSAGE);
      return;
    }
    if ("".equals(this.periodText.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A period name must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }

    ok = true;
    dispose();
  }

  private void onCancel() {
    ok = false;
    dispose();
  }

  public void setData(NewScheduledServiceAction data) {
    delayText.setText(data.getDelay());
    periodText.setText(data.getPeriod());
    javaPackageText.setText(data.getJavaPackage());
    serviceNameText.setText(data.getServiceName());
  }

  public void getData(NewScheduledServiceAction data) {
    data.setDelay(delayText.getText());
    data.setPeriod(periodText.getText());
    data.setJavaPackage(javaPackageText.getText());
    data.setServiceName(serviceNameText.getText());
  }

  public boolean isOk() {
    return ok;
  }
}
