/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.project.Project;
import midstream.core.handler.SocketClientHandler;
import midstream.util.IdeUtils;

import javax.swing.JOptionPane;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class NewSocketClientAction extends AnAction {
  private NewSocketClientDialog dialog;
  private String javaPackage;
  private String clientName;
  private boolean secure;
  private String host;
  private String port;
  private String connectionTimeout = "10";
  private String readTimeout = "10";
  private String keystoreLocation;
  private String keystoreType;
  private String keyAlgorithm;
  private String secureProtocol;

  public void update(AnActionEvent event) {
    Project project =
        (Project) event.getDataContext().getData(DataConstants.PROJECT);

    event.getPresentation().setVisible(IdeUtils.isPluginProject(project));
  }

  public void actionPerformed(AnActionEvent e) {
    Project project =
        (Project) e.getDataContext().getData(DataConstants.PROJECT);

    if (project == null) {
      JOptionPane.showMessageDialog(null, "The current project is invalid.",
          "Error Creating Socket Client", JOptionPane.ERROR_MESSAGE);
      return;
    }

    if (this.dialog == null) {
      this.dialog =
          new NewSocketClientDialog(IdeUtils.suggestParentWindow(project));
    }

    dialog.setData(this);
    dialog.setVisible(true);

    if (dialog.isOk()) {
      dialog.getData(this);

      try {
        final String projectPath = project.getProjectFilePath().substring(0,
            project.getProjectFilePath().indexOf(
                project.getProjectFile().getName()));

        Map<String, String> pageValues = new HashMap<String, String>(7);
        pageValues.put("java.package", getJavaPackage());
        pageValues.put("client.name", getClientName());
        pageValues.put("host", getHost());
        pageValues.put("port", getPort());
        pageValues.put("connection.timeout", getConnectionTimeout());
        pageValues.put("read.timeout", getReadTimeout());

        if (isSecure()) {
          pageValues.put("secure", "true");
          pageValues.put("keystore.location", getKeystoreLocation());
          pageValues.put("keystore.type", getKeystoreType());
          pageValues.put("key.algorithm", getKeyAlgorithm());
          pageValues.put("secure.protocol", getSecureProtocol());
        }

        SocketClientHandler socketClientHandler =
            new SocketClientHandler(projectPath);
        String clientClass =
            socketClientHandler.createClientClass(pageValues);
        socketClientHandler.createClientClassTest(pageValues);
        socketClientHandler.createContextEntry(pageValues);

        IdeUtils.reloadProject();
        IdeUtils
            .openFile(project, projectPath + File.separatorChar + clientClass);
      } catch (IOException ex) {
        JOptionPane.showMessageDialog(null, ex,
            "Error Creating Socket Client", JOptionPane.ERROR_MESSAGE);
        Logger.getInstance(this.getClass().getName()).error(ex);
      }
    }
  }

  public NewSocketClientDialog getDialog() {
    return dialog;
  }

  public void setDialog(NewSocketClientDialog dialog) {
    this.dialog = dialog;
  }

  public String getJavaPackage() {
    return javaPackage;
  }

  public void setJavaPackage(String javaPackage) {
    this.javaPackage = javaPackage;
  }

  public String getClientName() {
    return clientName;
  }

  public void setClientName(String clientName) {
    this.clientName = clientName;
  }

  public boolean isSecure() {
    return secure;
  }

  public void setSecure(boolean secure) {
    this.secure = secure;
  }

  public String getHost() {
    return host;
  }

  public void setHost(String host) {
    this.host = host;
  }

  public String getPort() {
    return port;
  }

  public void setPort(String port) {
    this.port = port;
  }

  public String getConnectionTimeout() {
    return connectionTimeout;
  }

  public void setConnectionTimeout(String connectionTimeout) {
    this.connectionTimeout = connectionTimeout;
  }

  public String getReadTimeout() {
    return readTimeout;
  }

  public void setReadTimeout(String readTimeout) {
    this.readTimeout = readTimeout;
  }

  public String getKeystoreLocation() {
    return keystoreLocation;
  }

  public void setKeystoreLocation(String keystoreLocation) {
    this.keystoreLocation = keystoreLocation;
  }

  public String getKeystoreType() {
    return keystoreType;
  }

  public void setKeystoreType(String keystoreType) {
    this.keystoreType = keystoreType;
  }

  public String getKeyAlgorithm() {
    return keyAlgorithm;
  }

  public void setKeyAlgorithm(String keyAlgorithm) {
    this.keyAlgorithm = keyAlgorithm;
  }

  public String getSecureProtocol() {
    return secureProtocol;
  }

  public void setSecureProtocol(String secureProtocol) {
    this.secureProtocol = secureProtocol;
  }
}
