/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import midstream.util.IdeUtils;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.table.DefaultTableModel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * @author Juan Hall
 */
public class NewSocketClientDialog extends JDialog {
  private JPanel contentPane;
  private JButton buttonOK;
  private JButton buttonCancel;
  private JTextField javaPackageText;
  private JTextField clientNameText;
  private JCheckBox secureCheckBox;
  private JTable connectionTable;
  private boolean ok;

  public NewSocketClientDialog(JFrame parent) {
    super(parent, "New Socket Client", true);
    setContentPane(contentPane);
    getRootPane().setDefaultButton(buttonOK);

    buttonOK.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onOK();
      }
    });

    buttonCancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    });

// call onCancel() when cross is clicked
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        onCancel();
      }
    });

// call onCancel() on ESCAPE
    contentPane.registerKeyboardAction(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
        JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

    DefaultCellEditor cellEditor = new DefaultCellEditor(new JTextField());
    cellEditor.setClickCountToStart(1);

    DefaultTableModel model = new DefaultTableModel() {
      public boolean isCellEditable(int row, int col) {
        return col > 0;
      }
    };

    model.addColumn("");
    model.addColumn("");
    model.addRow(new Object[]{"Host", ""});
    model.addRow(new Object[]{"Port", ""});
    model.addRow(new Object[]{"Connection Timeout", ""});
    model.addRow(new Object[]{"Read Timeout", ""});
    model.addRow(new Object[]{"Key Store Location", ""});
    model.addRow(new Object[]{"Key Store Type", ""});
    model.addRow(new Object[]{"Key Algorithm", ""});
    model.addRow(new Object[]{"Secure Protocol", ""});
    connectionTable.setModel(model);
    connectionTable.getColumnModel().getColumn(1).setCellEditor(cellEditor);

    IdeUtils.packAndCenter(this);
  }

  private void onOK() {
    if ("".equals(this.javaPackageText.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A Java package must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    if ("".equals(this.clientNameText.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A client name must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    if (IdeUtils.isTableEmpty(connectionTable, 1)) {
      JOptionPane.showMessageDialog(this,
          "Connection properties must be specified.", "Error",
          JOptionPane.ERROR_MESSAGE);
      return;
    }

    ok = true;
    dispose();
  }

  public void setData(NewSocketClientAction data) {
    javaPackageText.setText(data.getJavaPackage());
    clientNameText.setText(data.getClientName());
    secureCheckBox.setSelected(data.isSecure());

    int i = 0;
    connectionTable.setValueAt(data.getHost(), i++, 1);
    connectionTable.setValueAt(data.getPort(), i++, 1);
    connectionTable.setValueAt(data.getConnectionTimeout(), i++, 1);
    connectionTable.setValueAt(data.getReadTimeout(), i++, 1);
    connectionTable.setValueAt(data.getKeystoreLocation(), i++, 1);
    connectionTable.setValueAt(data.getKeystoreType(), i++, 1);
    connectionTable.setValueAt(data.getKeyAlgorithm(), i++, 1);
    connectionTable.setValueAt(data.getSecureProtocol(), i, 1);
  }

  public void getData(NewSocketClientAction data) {
    data.setJavaPackage(javaPackageText.getText());
    data.setClientName(clientNameText.getText());
    data.setSecure(secureCheckBox.isSelected());

    int i = 0;
    data.setHost((String) connectionTable.getValueAt(i++, 1));
    data.setPort((String) connectionTable.getValueAt(i++, 1));
    data.setConnectionTimeout((String) connectionTable.getValueAt(i++, 1));
    data.setReadTimeout((String) connectionTable.getValueAt(i++, 1));
    data.setKeystoreLocation((String) connectionTable.getValueAt(i++, 1));
    data.setKeystoreType((String) connectionTable.getValueAt(i++, 1));
    data.setKeyAlgorithm((String) connectionTable.getValueAt(i++, 1));
    data.setSecureProtocol((String) connectionTable.getValueAt(i, 1));
  }

  private void onCancel() {
    ok = false;
    dispose();
  }

  public boolean isOk() {
    return ok;
  }
}
