/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import midstream.util.IdeUtils;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

/**
 * @author Juan Hall
 */
public class NewXmlRpcClientDialog extends JDialog {
  private JPanel contentPane;
  private JButton buttonOK;
  private JButton buttonCancel;
  private JTextField javaPackageText;
  private JTextField clientNameText;
  private JTextField methodNameText;
  private JTextField urlText;
  private JTextField connectionTimeoutText;
  private JTextField readTimeoutText;
  private boolean ok;

  public NewXmlRpcClientDialog(JFrame parent) {
    super(parent, "New XML-RPC Client", true);
    setContentPane(contentPane);
    getRootPane().setDefaultButton(buttonOK);

    buttonOK.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onOK();
      }
    });

    buttonCancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    });

// call onCancel() when cross is clicked
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        onCancel();
      }
    });

// call onCancel() on ESCAPE
    contentPane.registerKeyboardAction(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        onCancel();
      }
    }, KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
        JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

    IdeUtils.packAndCenter(this);
  }

  private void onOK() {
    if ("".equals(this.javaPackageText.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A Java package must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    if ("".equals(this.clientNameText.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A client name must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    if ("".equals(this.methodNameText.getText().trim())) {
      JOptionPane
          .showMessageDialog(this, "A method name must be entered.", "Error",
              JOptionPane.ERROR_MESSAGE);
      return;
    }
    if ("".equals(this.urlText.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A URL name must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    if ("".equals(this.connectionTimeoutText.getText().trim())) {
      JOptionPane
          .showMessageDialog(this, "A connection timeout must be entered.",
              "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }
    if ("".equals(this.readTimeoutText.getText().trim())) {
      JOptionPane.showMessageDialog(this, "A read timeout must be entered.",
          "Error", JOptionPane.ERROR_MESSAGE);
      return;
    }

    ok = true;
    dispose();
  }

  public void setData(NewXmlRpcClientAction data) {
    javaPackageText.setText(data.getJavaPackage());
    clientNameText.setText(data.getClientName());
    methodNameText.setText(data.getMethodName());
    urlText.setText(data.getUrl());
    connectionTimeoutText.setText(data.getConnectionTimeout());
    readTimeoutText.setText(data.getReadTimeout());
  }

  public void getData(NewXmlRpcClientAction data) {
    data.setJavaPackage(javaPackageText.getText());
    data.setClientName(clientNameText.getText());
    data.setMethodName(methodNameText.getText());
    data.setUrl(urlText.getText());
    data.setConnectionTimeout(connectionTimeoutText.getText());
    data.setReadTimeout(readTimeoutText.getText());
  }

  private void onCancel() {
    ok = false;
    dispose();
  }

  public boolean isOk() {
    return ok;
  }
}
