/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.project.Project;
import midstream.core.util.WizardUtils;
import midstream.util.IdeUtils;

import javax.swing.JOptionPane;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Juan Hall
 */
public class ProjectDependenciesAction extends AnAction {
  private ProjectDependenciesDialog dialog;
  private List<String[]> dependencies;
  private int tab;
  private boolean addToBuildPath;
  private boolean addToDeploymentUnit;
  private boolean removeFromBuildPath;
  private boolean removeFromDeploymentUnit;

  public void update(AnActionEvent event) {
    Project project =
        (Project) event.getDataContext().getData(DataConstants.PROJECT);

    event.getPresentation().setVisible(IdeUtils.isPluginProject(project) &&
        event.getDataContext().getData(DataConstants.MODULE_CONTEXT) != null);
  }

  public void actionPerformed(AnActionEvent e) {
    Project project =
        (Project) e.getDataContext().getData(DataConstants.PROJECT);

    if (project == null) {
      JOptionPane.showMessageDialog(null, "The current project is invalid.",
          "Error Managing Project Dependencies", JOptionPane.ERROR_MESSAGE);
      return;
    }

    final String projectPath = project.getProjectFilePath().substring(0,
        project.getProjectFilePath().indexOf(
            project.getProjectFile().getName()));

    setDialog(
        new ProjectDependenciesDialog(
            IdeUtils.suggestParentWindow(project), projectPath));

    getDialog().setVisible(true);

    if (getDialog().isOk()) {
      getDialog().getData(this);
      boolean isAdd = getTab() == 0;
      boolean isUpdate = getTab() == 1;
      boolean isRemove = getTab() == 2;

      try {
        List<String> libs = new ArrayList<String>(5);

        for (String[] info : dependencies) {
          libs.add(info[1]);
        }

        if (isUpdate) {
          Map<String, Boolean[]> buildInfo = new HashMap<String, Boolean[]>(
              libs.size());

          for (String lib : libs) {
            boolean onBuildPath =
                IdeUtils.isLibraryOnBuildPath(projectPath, project, lib);
            boolean isBuildFileWarLibrary = WizardUtils
                .isBuildFileWarLibrary(projectPath, lib);

            buildInfo.put(lib, new Boolean[]{onBuildPath,
                isBuildFileWarLibrary});
          }

          IdeUtils.updateLibraryBuildPath(projectPath, project, libs
              .toArray(new String[0]), false);
          WizardUtils.updateBuildFileWarLibraries(projectPath, libs
              .toArray(new String[0]), false);
          WizardUtils
              .updateDependencies(projectPath, getDependencies(), false);

          for (String[] dependency : dependencies) {
            dependency[2] = dependency[3];
          }

          WizardUtils
              .updateDependencies(projectPath, getDependencies(), true);
          IdeUtils.runAntTarget(projectPath, "init");

          for (Map.Entry<String, Boolean[]> entry : buildInfo.entrySet()) {
            if (entry.getValue()[0]) {
              IdeUtils.updateLibraryBuildPath(projectPath, project, libs
                  .toArray(new String[0]), true);
            }

            if (entry.getValue()[1]) {
              WizardUtils.updateBuildFileWarLibraries(projectPath, libs
                  .toArray(new String[0]), true);
            }
          }
        } else {
          if (isAdd) {
            WizardUtils
                .updateDependencies(projectPath, getDependencies(), true);
            IdeUtils.runAntTarget(projectPath, "init");
          }

          if (isAdd && isAddToBuildPath()) {
            IdeUtils.updateLibraryBuildPath(projectPath, project, libs
                .toArray(new String[0]), true);
          } else if (isRemove && isRemoveFromBuildPath()) {
            IdeUtils.updateLibraryBuildPath(projectPath, project, libs
                .toArray(new String[0]), false);
          }

          if (isAdd && isAddToDeploymentUnit()) {
            WizardUtils.updateBuildFileWarLibraries(projectPath, libs
                .toArray(new String[0]), true);
          } else if (isRemove && isRemoveFromDeploymentUnit()) {
            WizardUtils.updateBuildFileWarLibraries(projectPath, libs
                .toArray(new String[0]), false);
          }

          if (isRemove) {
            WizardUtils
                .updateDependencies(projectPath, getDependencies(), false);
          }
        }

        IdeUtils.reloadProject();
      } catch (IOException ex) {
        JOptionPane.showMessageDialog(null, ex,
            "Error " + (isAdd ? "Adding" : isUpdate ? "Updating" : "Removing") +
                " Dependency",
            JOptionPane.ERROR_MESSAGE);
        Logger.getInstance(this.getClass().getName()).error(ex);
      }
    }
  }

  public ProjectDependenciesDialog getDialog() {
    return dialog;
  }

  public void setDialog(ProjectDependenciesDialog dialog) {
    this.dialog = dialog;
  }

  public List<String[]> getDependencies() {
    return dependencies;
  }

  public void setDependencies(List<String[]> dependencies) {
    this.dependencies = dependencies;
  }

  public boolean isAddToBuildPath() {
    return addToBuildPath;
  }

  public void setAddToBuildPath(boolean addToBuildPath) {
    this.addToBuildPath = addToBuildPath;
  }

  public boolean isAddToDeploymentUnit() {
    return addToDeploymentUnit;
  }

  public void setAddToDeploymentUnit(boolean addToDeploymentUnit) {
    this.addToDeploymentUnit = addToDeploymentUnit;
  }

  public int getTab() {
    return tab;
  }

  public void setTab(int tab) {
    this.tab = tab;
  }

  public boolean isRemoveFromBuildPath() {
    return removeFromBuildPath;
  }

  public void setRemoveFromBuildPath(boolean removeFromBuildPath) {
    this.removeFromBuildPath = removeFromBuildPath;
  }

  public boolean isRemoveFromDeploymentUnit() {
    return removeFromDeploymentUnit;
  }

  public void setRemoveFromDeploymentUnit(boolean removeFromDeploymentUnit) {
    this.removeFromDeploymentUnit = removeFromDeploymentUnit;
  }
}
