/*
 * Copyright 2006 Travelocity.com L.P. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package midstream.util;

import com.intellij.ide.util.projectWizard.JavaModuleBuilder;
import com.intellij.openapi.module.ModifiableModuleModel;
import com.intellij.openapi.module.ModuleManager;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.project.ProjectManager;
import com.intellij.openapi.projectRoots.ProjectJdk;
import com.intellij.openapi.projectRoots.ProjectJdkTable;
import com.intellij.openapi.roots.ModifiableRootModel;
import com.intellij.openapi.roots.ModuleRootManager;
import com.intellij.openapi.roots.ProjectRootManager;
import com.intellij.openapi.util.Pair;
import com.intellij.pom.java.LanguageLevel;
import midstream.NewProjectApplicationComponent;

import java.io.File;

/**
 * @author Juan Hall
 */
public final class ProjectActionUtils {

  private ProjectActionUtils() {
  }

  public static Project loadProject(final Project project) throws Exception {
    return ProjectManager.getInstance()
        .loadAndOpenProject(project.getProjectFilePath());
  }

  public static ProjectJdk setProjectJdk(
      NewProjectApplicationComponent component, Project project) throws
      Exception {
    ProjectJdk jdk =
        ProjectJdkTable.getInstance()
            .findJdk(component.getProjectJdk());

    if (jdk != null) {
      ProjectRootManager projectManager =
          ProjectRootManager.getInstance(project);
      projectManager.setProjectJdk(jdk);
      Class prm =
          Class.forName(
              "com.intellij.openapi.roots.ex.ProjectRootManagerEx");
      prm.getMethod("setLanguageLevel", LanguageLevel.class)
          .invoke(projectManager, LanguageLevel.JDK_1_5);
    }

    return jdk;
  }

  public static Project createProject(String projectLocationPrefix,
                                      NewProjectApplicationComponent component)
      throws Exception {
    Class pmi =
        Class.forName(
            "com.intellij.openapi.project.impl.ProjectManagerImpl");
    Object pmiInstance = pmi.getMethod("getInstanceEx").invoke(null);
    return (Project) pmi
        .getMethod("newProject", String.class, boolean.class,
            boolean.class)
        .invoke(pmiInstance,
            projectLocationPrefix + File.separatorChar +
                component.getProjectName() +
                ".ipr",
            true, false);
  }

  public static void createModule(Project project, ProjectJdk jdk,
                                  String projectLocationPrefix,
                                  String projectName) throws Exception {
    JavaModuleBuilder moduleBuilder = new JavaModuleBuilder();
    ModifiableModuleModel projectModel =
        ModuleManager.getInstance(project).getModifiableModel();
    moduleBuilder.setName(projectName);
    moduleBuilder
        .setModuleFilePath(projectLocationPrefix + File.separatorChar +
            projectName + ".iml");
    moduleBuilder.addSourcePath(new Pair<String, String>(
        projectLocationPrefix + File.separatorChar + "src", ""));

    moduleBuilder.setCompilerOutputPath(
        projectLocationPrefix + File.separatorChar + "build");
    moduleBuilder.setContentEntryPath(projectLocationPrefix);
    moduleBuilder.setModuleJdk(jdk);
    moduleBuilder.createModule(projectModel);
    projectModel.commit();

    ModifiableRootModel moduleModel = ModuleRootManager
        .getInstance(ModuleManager.getInstance(project).getModules()[0])
        .getModifiableModel();
    moduleModel.setCompilerOutputPathForTests(projectLocationPrefix +
        File.separatorChar + "test" + File.separatorChar + "build");
    moduleModel.getContentEntries()[0]
        .addSourceFolder(IdeUtils.getVirtualFile(projectLocationPrefix +
            File.separatorChar + "test" + File.separatorChar + "src"), true);

    moduleModel.commit();
  }
}
